package edu.ksu.cis.ndpc;

import edu.ksu.cis.ndpc.gui.GUIController;
import edu.ksu.cis.ndpc.gui.GUIHandler;
import edu.ksu.cis.ndpc.gui.GUIListener;
import edu.ksu.cis.ndpc.gui.GUIManager;
import edu.ksu.cis.ndpc.gui.RawProof;
import edu.ksu.cis.ndpc.io.IO;
import edu.ksu.cis.ndpc.io.InputFile;
import edu.ksu.cis.ndpc.io.Latex;
import edu.ksu.cis.ndpc.io.OutputFile;

import javax.imageio.ImageIO;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JScrollBar;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JToolBar;
import javax.swing.MenuElement;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.ImageIcon;
import javax.swing.BorderFactory;
import javax.swing.border.Border;

import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import java.io.File;
import java.io.IOException;

final public class Main {
	private static final String PRG_TITLE = "Natural Deduction Proof Checker";
	private static final String PRG_VERSION = "Version 0.2.2.0";
	private static final String PRG_COPYRIGHT = "Copyright  2007, Brian Mulanda, Rod Howell. All rights reserved.";

	private static final String ACTION_VIEW = "VIEW";
	private static final String ACTION_OPEN = "OPEN";
	private static final String ACTION_EXPORT = "EXPORT";
	private static final String ACTION_EXPORT_IMAGE = "EXPORT_IMAGE";

	private static final String ACTION_PRINT = "PRINT";
	private static final String ACTION_NEWPRED = "NEWPRED";
	private static final String ACTION_NEWPROP = "NEWPROP";
	private static final String ACTION_SAVE = "SAVE";
	private static final String ACTION_SAVEAS = "SAVEAS";
	private static final String ACTION_DRAWBOX = "DRAWBOX";
	private static final String ACTION_CHECK = "CHECK";
	private static final String ACTION_CLEAR = "CLEAR";
	private static final String ACTION_CLOSE = "CLOSE";
	private static final String ACTION_FONT = "FONT";
	private static final String ACTION_ABOUT = "ABOUT";

	private static final ImageIcon IMAGE_VIEW = Utils
			.createImageIcon("Find24.gif");
	private static final ImageIcon IMAGE_NEWPRED = Utils
			.createImageIcon("Add24.gif");
	private static final ImageIcon IMAGE_NEWPROP = Utils
			.createImageIcon("New24.gif");
	private static final ImageIcon IMAGE_OPEN = Utils
			.createImageIcon("Open24.gif");
	private static final ImageIcon IMAGE_SAVE = Utils
			.createImageIcon("Save24.gif");
	private static final ImageIcon IMAGE_SAVEAS = Utils
			.createImageIcon("SaveAs24.gif");
	private static final ImageIcon IMAGE_DRAWBOX = Utils
			.createImageIcon("DrawBox24.gif");
	private static final ImageIcon IMAGE_CLEAR = Utils
			.createImageIcon("Clear24.gif");
	private static final ImageIcon IMAGE_CHECK = Utils
			.createImageIcon("Check24.gif");
	private static final ImageIcon IMAGE_EXPORT = Utils
			.createImageIcon("Export24.gif");
	public static final ImageIcon IMAGE_ABOUT = Utils
			.createImageIcon("About4.gif");

	private static final int NO_MODIFIERS = 0;

	private static final JPanel BLANK_GUI = new JPanel();

	private ActionHandler actionHandler = new ActionHandler();
	private MenuHandler menuHandler = new MenuHandler();
	private WindowHandler windowHandler = new WindowHandler();
	private Border border = BorderFactory.createLineBorder(Color.black);
	private FlowLayout leftFlowLayout = new FlowLayout(FlowLayout.LEFT);

	private JTextArea messageArea;
	private JComboBox fontCombo;
	private JFrame frame;

	private File file;
	private JMenuBar menuBar;
	private JPopupMenu popupMenu;
	private JToolBar toolBar;

	private JScrollPane guiPane;

	private boolean proof_exists;
	private boolean proof_modified;
	private boolean clear_area;

	private StringBuffer frame_title;
	private int proof_type;

	private JScrollBar vScrollBar;
	private JScrollBar hScrollBar;
	private GUIController guiController;

	public Main() {
		this.frame = createJFrame();
		this.proof_exists = false;
		this.proof_modified = false;
		this.clear_area = true;
		this.frame_title = new StringBuffer();
		this.file = null;

		GUIListener l = new GUIListener(new GUIEventHandler());
		MouseListener mListener = new PopupListener();

		BLANK_GUI.addMouseListener(mListener);

		GUIController.init();
		GUIController.setGUIManager(new GUIFeedbackManager());
		GUIController.addMouseListener(mListener);
		GUIController.addMouseListener(l);
		GUIController.addMouseMotionListener(l);
		GUIController.addKeyListener(l);

		setEnabled();
		setTitle(true);

		frame.setVisible(true);
	}

	private JFrame createJFrame() {
		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();
		Point centerPoint = ge.getCenterPoint();
		Rectangle windowBounds = ge.getMaximumWindowBounds();
		Dimension frameSize = new Dimension(700, 600);

		if (frameSize.width > windowBounds.width) {
			frameSize.width = windowBounds.width;
		}

		if (frameSize.height > windowBounds.height) {
			frameSize.height = windowBounds.height;
		}

		Point frameLocation = new Point(centerPoint.x - (frameSize.width / 2),
				centerPoint.y - (frameSize.height / 2));

		JFrame frame = new JFrame();
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.setSize(frameSize);
		frame.setLocation(frameLocation);
		frame.addWindowListener(windowHandler);

		/*
		 * 
		 * The split pane occupies the entire frame area
		 */

		JSplitPane splitPane = createSplitPane(frameSize);

		/*
		 * It seems the splitPane is actually allocated a height less 100 than
		 * the frameSize.height. I don't understand how that works.
		 * 
		 * You should notice this behavior when you experiment with
		 * setDividerLocation.
		 * 
		 * I want to set the divider location such that the top component
		 * occupies 75% of the available height
		 */
		int divLoc = (int) Math.floor((frameSize.height - 100) * 0.65);
		splitPane.setDividerLocation(divLoc);

		Dimension proofAreaSize = new Dimension(frameSize.width, divLoc);
		Dimension messageAreaSize = new Dimension(frameSize.width,
				frameSize.height - divLoc);

		splitPane.setTopComponent(createProofArea(proofAreaSize));
		splitPane.setBottomComponent(createMessageArea(messageAreaSize));

		frame.setJMenuBar(createMenuBar());
		createPopupMenu();
		frame.getContentPane().add(createToolBar(), BorderLayout.PAGE_START);
		frame.getContentPane().add(splitPane, BorderLayout.CENTER);

		return frame;
	}

	private JSplitPane createSplitPane(Dimension size) {
		JSplitPane splitPane = new JSplitPane();
		splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
		splitPane.setContinuousLayout(true);
		splitPane.setOneTouchExpandable(true);
		splitPane.setOpaque(true);
		splitPane.setDividerSize(10);
		splitPane.setPreferredSize(size);
		return splitPane;
	}

	private Component createProofArea(Dimension d) {
		vScrollBar = new JScrollBar(JScrollBar.VERTICAL);
		hScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);

		guiPane = new JScrollPane();
		guiPane.setViewportView(BLANK_GUI);
		guiPane.setPreferredSize(d);
		guiPane.setFocusable(false);
		guiPane.setVerticalScrollBar(vScrollBar);
		guiPane.setHorizontalScrollBar(hScrollBar);

		return guiPane;
	}

	private Component createMessageArea(Dimension d) {
		JScrollPane messagePane = new JScrollPane();

		JTextArea textArea = new JTextArea(7, 2);
		textArea.setAutoscrolls(true);
		textArea.setWrapStyleWord(true);
		textArea.setLineWrap(true);
		textArea.setFont(new Font("Monospaced", 0, 14));
		textArea.setFocusable(false);
		textArea.setEditable(false);

		messagePane.setViewportView(textArea);
		messagePane.setPreferredSize(d);

		messageArea = textArea;

		return messagePane;
	}

	private JMenuBar createMenuBar() {
		menuBar = new JMenuBar();

		JMenu menuFile = createMenu("File", KeyEvent.VK_F, "");
		JMenu menuProof = createMenu("Proof", KeyEvent.VK_P, "");
		JMenu menuNew = createMenu("New", KeyEvent.VK_N, "New");
		JMenu menuHelp = createMenu("Help", KeyEvent.VK_H, "");
		JMenu menuExport = createMenu("Export", KeyEvent.VK_E, "");
		JMenuItem menuItem;

		menuFile.add(menuNew);
		menuItem = createMenuItem("Propositional Logic proof", IMAGE_NEWPROP,
				KeyEvent.VK_P, KeyStroke.getKeyStroke(KeyEvent.VK_P,
						ActionEvent.CTRL_MASK), ACTION_NEWPROP,
				"New Propositional Logic Proof");
		menuNew.add(menuItem);

		menuItem = createMenuItem("Predicate Logic proof", IMAGE_NEWPRED,
				KeyEvent.VK_D, KeyStroke.getKeyStroke(KeyEvent.VK_D,
						ActionEvent.CTRL_MASK), ACTION_NEWPRED,
				"New Predicate Logic Proof");

		menuNew.add(menuItem);

		menuItem = createMenuItem("Open...", IMAGE_OPEN, KeyEvent.VK_O,
				KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK),
				ACTION_OPEN, "Open Proof");
		menuFile.add(menuItem);

		menuFile.addSeparator();

		menuItem = createMenuItem("Save", IMAGE_SAVE, KeyEvent.VK_S, KeyStroke
				.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK),
				ACTION_SAVE, "Save Proof");
		menuFile.add(menuItem);

		menuItem = createMenuItem("Save As...", IMAGE_SAVEAS, KeyEvent.VK_A,
				KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK),
				ACTION_SAVEAS, "Save Proof As");
		menuFile.add(menuItem);

		menuFile.addSeparator();

		menuFile.add(menuExport);
		menuItem = createMenuItem("LaTeX...", null, KeyEvent.VK_E, KeyStroke
				.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK),
				ACTION_EXPORT, "Export to LaTeX");
		menuExport.add(menuItem);
		menuItem = createMenuItem("Image...", null, KeyEvent.VK_E, KeyStroke
				.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK),
				ACTION_EXPORT_IMAGE, "Export to image");
		menuExport.add(menuItem);

		menuFile.addSeparator();

		menuItem = createMenuItem("Print...", IMAGE_EXPORT, KeyEvent.VK_P,
				KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK),
				ACTION_PRINT, "Print Proof");
		menuFile.add(menuItem);

		menuFile.addSeparator();

		menuItem = createMenuItem("Close", null, KeyEvent.VK_C, KeyStroke
				.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK),
				ACTION_CLOSE, "Close");

		menuFile.add(menuItem);

		menuItem = createMenuItem("Draw Box...", IMAGE_DRAWBOX, KeyEvent.VK_B,
				KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK),
				ACTION_DRAWBOX, "Draw Box");
		menuProof.add(menuItem);

		menuItem = createMenuItem("Clear Message Area", IMAGE_CLEAR,
				KeyEvent.VK_F3, KeyStroke.getKeyStroke(KeyEvent.VK_F3,
						NO_MODIFIERS), ACTION_CLEAR, "Clear Message Area");
		menuProof.add(menuItem);

		/*
		 * menuProof.addSeparator();
		 * 
		 * menuItem = createMenuItem("View Tree", IMAGE_VIEW, KeyEvent.VK_T,
		 * KeyStroke.getKeyStroke(KeyEvent.VK_F6, NO_MODIFIERS), ACTION_VIEW,
		 * "Check Proof"); menuProof.add(menuItem);
		 */

		menuProof.addSeparator();

		menuItem = createMenuItem("Check", IMAGE_CHECK, KeyEvent.VK_K,
				KeyStroke.getKeyStroke(KeyEvent.VK_F5, NO_MODIFIERS),
				ACTION_CHECK, "Check Proof");
		menuProof.add(menuItem);

		menuItem = createMenuItem("About Natural Deduction Proof Checker",
				null, KeyEvent.VK_A, null, ACTION_ABOUT,
				"About Natural Deduction Proof Checker");
		menuHelp.add(menuItem);

		menuBar.add(menuFile);
		menuBar.add(menuProof);
		menuBar.add(menuHelp);

		addMenuListener(menuBar, actionHandler, menuHandler);

		return menuBar;
	}

	private JMenu createMenu(String text, int mnemonic, String accessDescr) {
		JMenu menu = new JMenu(text);
		menu.setMnemonic(mnemonic);
		menu.getAccessibleContext().setAccessibleDescription(accessDescr);

		return menu;
	}

	private JMenuItem createMenuItem(String text, ImageIcon image,
			int mnemonic, KeyStroke keyStroke, String actionCmd,
			String accessDescr) {

		JMenuItem menuItem = new JMenuItem(text, image);
		menuItem.setMnemonic(mnemonic);
		menuItem.setAccelerator(keyStroke);
		menuItem.setActionCommand(actionCmd);
		menuItem.getAccessibleContext().setAccessibleDescription(accessDescr);

		return menuItem;
	}

	private void createPopupMenu() {
		popupMenu = new JPopupMenu();
		JMenuItem menuItem;
		JMenu menuNew = createMenu("New", KeyEvent.VK_N, "New");

		menuItem = createMenuItem("Propositional Logic proof", IMAGE_NEWPROP,
				KeyEvent.VK_P, KeyStroke.getKeyStroke(KeyEvent.VK_P,
						ActionEvent.CTRL_MASK), ACTION_NEWPROP,
				"New Propositional Logic Proof");
		menuNew.add(menuItem);

		menuItem = createMenuItem("Predicate Logic proof", IMAGE_NEWPRED,
				KeyEvent.VK_D, KeyStroke.getKeyStroke(KeyEvent.VK_D,
						ActionEvent.CTRL_MASK), ACTION_NEWPRED,
				"New Predicate Logic Proof");
		menuNew.add(menuItem);

		popupMenu.add(menuNew);

		menuItem = createMenuItem("Open...", IMAGE_OPEN, KeyEvent.VK_O,
				KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK),
				ACTION_OPEN, "Open Proof");
		popupMenu.add(menuItem);

		menuItem = createMenuItem("Export to LaTeX...", IMAGE_EXPORT,
				KeyEvent.VK_E, KeyStroke.getKeyStroke(KeyEvent.VK_E,
						ActionEvent.CTRL_MASK), ACTION_EXPORT,
				"Export to LaTeX");
		popupMenu.add(menuItem);

		popupMenu.addSeparator();

		menuItem = createMenuItem("Draw Box...", IMAGE_DRAWBOX, KeyEvent.VK_B,
				KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK),
				ACTION_DRAWBOX, "Draw Box");
		popupMenu.add(menuItem);

		menuItem = createMenuItem("Clear Message Area", IMAGE_CLEAR,
				KeyEvent.VK_F3, KeyStroke.getKeyStroke(KeyEvent.VK_F3,
						NO_MODIFIERS), ACTION_CLEAR, "Clear Message Area");
		popupMenu.add(menuItem);

		/*
		 * popupMenu.addSeparator();
		 * 
		 * menuItem = createMenuItem("View Tree", IMAGE_VIEW, KeyEvent.VK_T,
		 * KeyStroke.getKeyStroke(KeyEvent.VK_F6, NO_MODIFIERS), ACTION_VIEW,
		 * "Check Proof"); popupMenu.add(menuItem);
		 */

		popupMenu.addSeparator();

		menuItem = createMenuItem("Check Proof", IMAGE_CHECK, KeyEvent.VK_K,
				KeyStroke.getKeyStroke(KeyEvent.VK_F5, NO_MODIFIERS),
				ACTION_CHECK, "Check Proof");
		popupMenu.add(menuItem);

		addMenuListener(popupMenu, actionHandler, menuHandler);
	}

	private void addMenuListener(MenuElement menuElem,
			ActionListener aListener, MenuListener mListener) {
		MenuElement[] elements = menuElem.getSubElements();
		if (elements.length > 0) {
			for (int i = 0; i < elements.length; ++i) {
				addMenuListener(elements[i], aListener, mListener);
			}
		}

		if (menuElem instanceof JMenu) {
			JMenu menu = (JMenu) menuElem;
			if (menu.isTopLevelMenu()) {
				menu.addMenuListener(mListener);
			}
		}

		if (menuElem instanceof JMenuItem) {
			((JMenuItem) menuElem).addActionListener(aListener);
		}
	}

	private JToolBar createToolBar() {
		toolBar = new JToolBar();
		JButton button;

		button = createToolbarButton(IMAGE_NEWPROP, ACTION_NEWPROP,
				"New Propositional Logic proof", actionHandler, border);
		toolBar.add(button);

		button = createToolbarButton(IMAGE_NEWPRED, ACTION_NEWPRED,
				"New Predicate Logic proof", actionHandler, border);
		toolBar.add(button);

		button = createToolbarButton(IMAGE_OPEN, ACTION_OPEN,
				"Open existing proof", actionHandler, border);
		toolBar.add(button);

		toolBar.addSeparator();

		button = createToolbarButton(IMAGE_SAVE, ACTION_SAVE, "Save proof",
				actionHandler, border);
		toolBar.add(button);

		button = createToolbarButton(IMAGE_EXPORT, ACTION_EXPORT,
				"Export to LaTeX", actionHandler, border);
		toolBar.add(button);

		toolBar.addSeparator();

		button = createToolbarButton(IMAGE_DRAWBOX, ACTION_DRAWBOX, "Draw Box",
				actionHandler, border);
		toolBar.add(button);

		button = createToolbarButton(IMAGE_CLEAR, ACTION_CLEAR,
				"Clear Message Area", actionHandler, border);
		toolBar.add(button);

		toolBar.addSeparator();

		button = createToolbarButton(IMAGE_CHECK, ACTION_CHECK, "Check proof",
				actionHandler, border);
		toolBar.add(button);

		/*
		 * toolBar.addSeparator();
		 * 
		 * button = createToolbarButton( IMAGE_VIEW, ACTION_VIEW,
		 * "View Proof Tree", actionHandler, border); toolBar.add(button);
		 */

		int[] fts = { 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22 };

		Integer[] fonts = new Integer[fts.length];
		for (int i = 0; i < fts.length; ++i) {
			fonts[i] = new Integer(fts[i]);
		}
		fontCombo = new JComboBox(fonts);
		fontCombo.setEditable(false);
		fontCombo.setActionCommand(ACTION_FONT);
		fontCombo.addActionListener(actionHandler);
		fontCombo.setSize(new Dimension(40, fontCombo.getHeight()));
		fontCombo.setSelectedItem(new Integer(16));

		JLabel fontLabel = new JLabel("Font size: ");
		JPanel fontPanel = new JPanel(leftFlowLayout);
		fontPanel.add(fontLabel);
		fontPanel.add(fontCombo);
		toolBar.add(fontPanel);

		toolBar.setFloatable(true);
		toolBar.setRollover(true);

		return toolBar;
	}

	private JButton createToolbarButton(ImageIcon image, String action,
			String toolTipText, ActionListener listener, Border border) {

		return createToolbarButton(image, null, action, toolTipText, listener,
				border);
	}

	private JButton createToolbarButton(ImageIcon image, String altText,
			String action, String toolTipText, ActionListener listener,
			Border border) {

		JButton button = new JButton();
		if (image != null) {
			button.setIcon(image);
		} else {
			button.setText(altText);
		}
		if (action != null) {
			button.setActionCommand(action);
		}
		button.setToolTipText(toolTipText);
		button.addActionListener(listener);
		button.setBorder(border);

		return button;
	}

	private void setEnabled() {
		setEnabled((MenuElement) menuBar);
		setEnabled((MenuElement) popupMenu);
		setEnabled(toolBar);
	}

	private void setEnabled(Component component) {
		if (component instanceof MenuElement) {
			setEnabled((MenuElement) component);
		} else if (component instanceof JButton) {
			JButton button = (JButton) component;
			button.setEnabled(getEnabled(button.getActionCommand()));
		} else if (component instanceof JComboBox) {
			JComboBox comboBox = (JComboBox) component;
			comboBox.setEnabled(getEnabled(comboBox.getActionCommand()));
		} else if (component instanceof Container) {
			setEnabled(((Container) component).getComponents());
		}
	}

	private void setEnabled(Component[] components) {
		for (int i = 0; i < components.length; ++i) {
			setEnabled(components[i]);
		}
	}

	private void setEnabled(MenuElement menuElem) {
		MenuElement[] elements = menuElem.getSubElements();
		if (elements.length > 0) {
			for (int i = 0; i < elements.length; ++i) {
				setEnabled(elements[i]);
			}
		}

		if (menuElem instanceof JMenuItem) {
			JMenuItem menuItem = (JMenuItem) menuElem;
			menuItem.setEnabled(getEnabled(menuItem.getActionCommand()));
		}
	}

	private boolean getEnabled(String actionCmd) {
		boolean enabled = true;
		if (actionCmd.equals(ACTION_NEWPROP)) {
			// always true
			enabled = true;
		} else if (actionCmd.equals(ACTION_NEWPRED)) {
			// always true
			enabled = true;
		} else if (actionCmd.equals(ACTION_OPEN)) {
			// always true
			enabled = true;
		} else if (actionCmd.equals(ACTION_SAVE)) {
			enabled = (proof_exists && proof_modified);
		} else if (actionCmd.equals(ACTION_SAVEAS)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_PRINT)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_CLOSE)) {
			// always true
			enabled = true;
		} else if (actionCmd.equals(ACTION_EXPORT)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_EXPORT_IMAGE)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_CHECK)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_CLEAR)) {
			enabled = (proof_exists && clear_area);
		} else if (actionCmd.equals(ACTION_DRAWBOX)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_FONT)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_VIEW)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_FONT)) {
			enabled = (proof_exists);
		} else if (actionCmd.equals(ACTION_ABOUT)) {
			// always true
		}
		return enabled;
	}

	private void check() {
		RawProof rawProof = guiController.getProof();

		ProofChecker pc = new ProofChecker();
		try {
			if (rawProof.getType() == RawProof.PROOF_PRED) {
				pc.check(rawProof, new PredLParser());
			} else if (rawProof.getType() == RawProof.PROOF_PROP) {
				pc.check(rawProof, new PropLParser());
			}
			messageArea.setText("Proof: OK");
		} catch (Exception pce) {
			// pce.printStackTrace();
			messageArea.setText(pce.getMessage());
		}
	}

	private void clear() {
		messageArea.setText("");
	}

	private void close() {
		if (canCloseProof()) {
			System.exit(0);
		}
	}

	private boolean canCloseProof() {
		boolean closed = true;
		if (proof_exists && proof_modified) {
			String message;
			if (file == null) {
				message = "Save proof to file?";
			} else {
				message = "Save changes to proof?";
			}
			int response = JOptionPane.showConfirmDialog(frame, message,
					"Confirm", JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE);
			if (response == JOptionPane.YES_OPTION) {
				if (saveProof()) {
					proof_modified = false;
					file = null;
				} else {
					closed = false;
				}
			} else if (response == JOptionPane.NO_OPTION) {
				proof_modified = false;
			} else { // user selected CANCEL or Closed the dialog box
				closed = false;
			}
		}
		return closed;
	}

	private void view() {
		messageArea.setText("View Proof Tree");
		RawProof rawProof = guiController.getProof();
		ProofChecker pc = new ProofChecker();
		try {
			Proof proof = null;
			if (rawProof.getType() == RawProof.PROOF_PRED) {
				proof = pc.buildProof(rawProof);
			} else {
				proof = pc.buildProof(rawProof);
			}
			new ProofTreeViewer().viewTree(proof.getTree());
			messageArea.setText("View proof tree");
		} catch (Exception pce) {
			messageArea.setText(pce.getMessage());
		}
	}

	private void newProof(int type) {
		if (!proof_exists || (proof_exists && canCloseProof())) {
			RawProof proof = new RawProof(type);
			proof.addStatement(null);
			setProof(proof);
			proof_exists = true;
			proof_modified = false;
			proof_type = type;
			file = null;
		}
	}

	private void setProof(RawProof proof) {
		JComponent comp = BLANK_GUI;
		if (proof != null) {
			guiController = GUIController.setProof(proof);
			if (guiController != null) {
				comp = guiController.getGUI();
			}
		}
		guiPane.setViewportView(comp);
		comp.scrollRectToVisible(comp.getVisibleRect());
		messageArea.setText("");
	}

	private void openProof() {
		messageArea.setText("Opening proof...");
		InputFile infile = IO.openInputFile(frame);
		if (infile != null) {
			if (!proof_exists || (proof_exists && canCloseProof())) {
				file = infile.getFile();
				try {
					RawProof proof = null;
					Object data = IO.readFromFile(infile);
					if (IO.getType(infile) == IO.TYPE_OBJECT) {
						proof = (RawProof) data;
					} else if (IO.getType(infile) == IO.TYPE_TEXT) {
						proof = RawProof.fromString((String) data);
					}

					setProof(proof);
					proof_exists = true;
					proof_type = proof.getType();
					messageArea.setText("Loaded proof from file: "
							+ file.getAbsolutePath());
				} catch (Exception e) {
					setProof(null);
					proof_exists = false;
					messageArea.setText("Could not load proof\n "
							+ file.getAbsolutePath()
							+ " does not seem to be a valid proof file");
				}
			}
			infile.close();
		} else {
			messageArea.setText("");
		}
	}

	private boolean saveProof() {
		messageArea.setText("Saving proof...");
		boolean saved = false;
		if (file == null) {
			saved = saveProofAs();
		} else {
			if (proof_exists && proof_modified) {
				OutputFile outfile = IO.openOutputFile(file);
				if (outfile != null) {
					try {
						RawProof proof = guiController.getProof();
						IO.writeToFile(outfile, proof);
						outfile.close();
						proof_modified = false;
						messageArea.setText("Saved changes to "
								+ file.getAbsolutePath());
						saved = true;
					} catch (Exception ioe) {
						messageArea.setText("Error while saving proof\n"
								+ ioe.toString());
					}
				} else {
					messageArea.setText("Could not open file: "
							+ file.getAbsolutePath());
				}
			} else {
				messageArea.setText("Nothing to Save");
			}
		}
		return saved;
	}

	private boolean saveProofAs() {
		messageArea.setText("Saving proof...");
		boolean saved = false;
		if (proof_exists) {
			try {
				OutputFile outfile = IO.openOutputFile(frame);
				if (outfile != null) {
					file = outfile.getFile();
					RawProof proof = guiController.getProof();
					IO.writeToFile(outfile, proof);
					outfile.close();
					proof_modified = false;
					messageArea.setText("Saved proof to file: "
							+ file.getAbsolutePath());
					saved = true;
				} else {
					messageArea.setText("");
				}
			} catch (Exception ioe) {
				messageArea.setText("Error while saving proof\n"
						+ ioe.toString());
			}
		} else {
			messageArea.setText("Nothing to Save As");
		}
		return saved;
	}

	private void exportLatex() {
		messageArea.setText("Exporting proof...");
		try {
			java.io.File file = Latex.export(guiController.getProof());
			if (file != null) {
				messageArea.setText("Exported proof to file: "
						+ file.getAbsolutePath());
			} else {
				messageArea.setText("");
			}
		} catch (Exception ioe) {
			messageArea.setText(ioe.toString());
		}
	}

	private void exportImage() {
		messageArea.setText("Exporting proof...");

		if (proof_exists) {
			try {
				OutputFile outfile = IO.openImageOutputFile(frame);
				if (outfile != null) {
					file = outfile.getFile();
					String format = file.getName().substring(
							file.getName().lastIndexOf('.') + 1);
					ImageIO.write(guiController.getImage(), format, file);
					outfile.close();
					messageArea.setText("Exported image to file: "
							+ file.getAbsolutePath() + " " + file.getName());
				} else {
					messageArea.setText("");
				}
			} catch (Exception ioe) {
				messageArea.setText("Error while exporting proof as image\n"
						+ ioe.toString());
			}
		} else {
			messageArea.setText("Nothing to Save As");
		}
	}

	private void printProof() {
		messageArea.setText("Printing proof...");

		PrinterJob printJob = PrinterJob.getPrinterJob();
		PageFormat format = printJob.defaultPage();
		format.setOrientation(PageFormat.PORTRAIT);

		printJob.setPrintable(new Printable() {
			public int print(Graphics pg, PageFormat pf, int page) {
				if (page > 0) {
					return Printable.NO_SUCH_PAGE;
				}

				Graphics2D g2 = (Graphics2D) pg;
				double scale = Math.min(pf.getImageableHeight()
						/ guiController.getGUI().getHeight(), pf
						.getImageableWidth()
						/ guiController.getGUI().getWidth());

//				System.out.println("pf H " + pf.getImageableHeight());
//				System.out.println("pf W " + pf.getImageableWidth());
//				System.out
//						.println("gui H " + guiController.getGUI().getWidth());
//				System.out.println("gui W "
//						+ guiController.getGUI().getHeight());

				g2.setPaint(Color.black);
				g2.translate(pf.getImageableX(), pf.getImageableY());
				g2.scale(scale, scale);
				// guiController.getGUI().paint(g2);
				guiController.getGUI().printAll(g2);
				return Printable.PAGE_EXISTS;
			}
		});

		if (printJob.printDialog() == false)
			return;

		try {
			printJob.print();
		} catch (PrinterException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		messageArea.setText("");
	}

	private void drawBox() {
		guiController.pauseCursor();
		BoxDialog boxDialog = new BoxDialog(frame);
		boxDialog.setVisible(true);
		if (!boxDialog.canceled()) {
			int startRow = boxDialog.getStartRow();
			int endRow = boxDialog.getEndRow();
			if (startRow > endRow) {
				int temp = startRow;
				startRow = endRow;
				endRow = temp;
			}
			if (!guiController.drawBox(startRow - 1, endRow, true)) {
				messageArea.setText("Cannot draw box: Rows[" + (startRow) + ","
						+ (endRow) + "]\n");
			}
		}
		boxDialog.dispose();
		boxDialog = null;
		guiController.resumeCursor();
	}

	private void changeFontSize() {
		try {
			Object fontInt = fontCombo.getSelectedItem();
			if (fontInt != null) {
				int fontsize = ((Integer) fontInt).intValue();
				guiController.setFontsize(fontsize);
			}
		} catch (Exception e) {
		}
	}

	private void about() {
		new AboutBox(frame).setVisible(true);
	}

	private void setModified(boolean value) {
		if (value && !proof_modified) {
			proof_modified = value;
			setTitle();
			setEnabled();
		}
	}

	private StringBuffer createTitle() {
		StringBuffer title = new StringBuffer();
		if (proof_exists) {
			title.append(RawProof.getTypeString(proof_type)).append(" - ");
			if (file != null) {
				title.append(file.getName());
			} else {
				title.append("Untitled");
			}
			title.append(" - ");
		}
		title.append(PRG_TITLE).append("*");
		return title;
	}

	private void setTitle() {
		setTitle(false);
	}

	private void setTitle(boolean create) {
		if (create) {
			StringBuffer title = createTitle();
			if (frame_title.length() > 0) {
				frame_title.delete(0, frame_title.length());
			}
			frame_title.append(title);
		}
		if (proof_modified) {
			frame.setTitle(frame_title.substring(0, frame_title.length()));
		} else {
			frame.setTitle(frame_title.substring(0, frame_title.length() - 1));
		}
	}

	class AboutBox extends JDialog {
		AboutBox(JFrame aFrame) {
			super(aFrame, "About " + PRG_TITLE, true);
			addComponents();
			setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			setResizable(false);
			pack();
			setSize(getPreferredSize());
			setLocationRelativeTo(aFrame);
		}

		private void addComponents() {
			GridBagLayout gridbag = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();

			c.insets.top = 1;
			c.insets.left = 1;
			c.fill = GridBagConstraints.BOTH;
			c.weightx = 1.0;
			c.gridwidth = 1;
			c.gridheight = 4;

			JLabel labelImage = new JLabel(IMAGE_ABOUT);
			labelImage.setBorder(border);

			gridbag.setConstraints(labelImage, c);

			c.insets.left = 0;
			c.insets.right = 1;
			c.weighty = 0.0;
			c.gridwidth = GridBagConstraints.REMAINDER;
			c.gridheight = 4;

			JTextArea aboutText = new JTextArea();
			StringBuffer aboutStr = new StringBuffer(100);
			aboutStr.append("\n").append("  ").append(PRG_TITLE).append("\n")
					.append("  ").append(PRG_VERSION).append("\n\n").append(
							"  ").append(PRG_COPYRIGHT).append(" \n");
			aboutText.setText(aboutStr.toString());
			aboutText.setEditable(false);
			aboutText.setFocusable(false);

			gridbag.setConstraints(aboutText, c);

			JButton aboutButton = new JButton("OK");
			aboutButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					AboutBox.this.dispose();
				}
			});

			c.insets.top = 5;
			c.insets.bottom = 5;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.LAST_LINE_END;
			c.gridwidth = 2;
			gridbag.setConstraints(aboutButton, c);

			getContentPane().add(labelImage);
			getContentPane().add(aboutText);
			getContentPane().add(aboutButton);
			getContentPane().setLayout(gridbag);
		}
	}

	class BoxDialog extends JDialog implements ActionListener {

		static final long serialVersionUID = 1;
		private int startRow = -1;
		private int endRow = -1;

		private JTextField startRowField;
		private JTextField endRowField;

		private JButton buttonDraw;
		private JButton buttonCancel;

		private boolean canceled = true;

		public BoxDialog(JFrame aFrame) {
			super(aFrame, "Draw Box", true);

			setSize(400, 100);
			setLocationRelativeTo(aFrame);
			setResizable(false);
			startRowField = new JTextField(2);
			endRowField = new JTextField(2);

			JPanel panel = new JPanel(leftFlowLayout);
			panel.add(new JLabel("Start Row:"));
			panel.add(startRowField);
			panel.add(new JLabel("  End Row:"));
			panel.add(endRowField);

			buttonDraw = new JButton("Draw");
			buttonCancel = new JButton("Cancel");
			panel.add(buttonDraw);
			panel.add(buttonCancel);

			buttonDraw.addActionListener(this);
			buttonCancel.addActionListener(this);

			setContentPane(panel);
		}

		public int getStartRow() {
			return startRow;
		}

		public int getEndRow() {
			return endRow;
		}

		public boolean canceled() {
			return canceled;
		}

		public void actionPerformed(ActionEvent e) {
			try {
				if (e.getSource() == buttonDraw) {
					startRow = Integer.parseInt(startRowField.getText());
					endRow = Integer.parseInt(endRowField.getText());
					canceled = false;
				} else if (e.getSource() == buttonCancel) {
					canceled = true;
				}
				setVisible(false);
			} catch (Exception excp) {

			}
		}
	}

	class WindowHandler extends WindowAdapter {
		public void windowClosing(WindowEvent we) {
			close();
		}
	}

	class ActionHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			boolean setEnabled = true;
			boolean setTitle = true;
			String actionCmd = e.getActionCommand();

			if (actionCmd.equals(ACTION_NEWPROP)) {
				newProof(RawProof.PROOF_PROP);
			} else if (actionCmd.equals(ACTION_NEWPRED)) {
				newProof(RawProof.PROOF_PRED);
			} else if (actionCmd.equals(ACTION_OPEN)) {
				openProof();
			} else if (actionCmd.equals(ACTION_SAVE)) {
				setTitle = setEnabled = saveProof();
			} else if (actionCmd.equals(ACTION_SAVEAS)) {
				setTitle = setEnabled = saveProofAs();
			} else if (actionCmd.equals(ACTION_CLOSE)) {
				close();
			} else if (actionCmd.equals(ACTION_EXPORT)) {
				exportLatex();
				setTitle = setEnabled = false;
			} else if (actionCmd.equals(ACTION_EXPORT_IMAGE)) {
				exportImage();
			} else if (actionCmd.equals(ACTION_PRINT)) {
				printProof();
			} else if (actionCmd.equals(ACTION_CHECK)) {
				check();
				setTitle = setEnabled = false;
			} else if (actionCmd.equals(ACTION_CLEAR)) {
				clear();
				setTitle = setEnabled = false;
			} else if (actionCmd.equals(ACTION_DRAWBOX)) {
				drawBox();
			} else if (actionCmd.equals(ACTION_FONT)) {
				changeFontSize();
				setTitle = setEnabled = false;
			} else if (actionCmd.equals(ACTION_VIEW)) {
				view();
				setTitle = setEnabled = false;
			} else if (actionCmd.equals(ACTION_ABOUT)) {
				about();
				setTitle = setEnabled = false;
			}

			if (setEnabled)
				setEnabled();
			if (setTitle)
				setTitle(true);
			if (guiController != null)
				guiController.getGUI().requestFocusInWindow();
		}
	}

	class PopupListener extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
			maybeShowPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			maybeShowPopup(e);
		}

		private void maybeShowPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popupMenu.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}

	class MenuHandler implements MenuListener {
		public void menuSelected(MenuEvent e) {
			pause();
		}

		public void menuDeselected(MenuEvent e) {
			resume();
		}

		public void menuCanceled(MenuEvent e) {
			resume();
		}

		private void resume() {
			if (guiController != null)
				guiController.resumeCursor();
		}

		private void pause() {
			if (guiController != null)
				guiController.pauseCursor();
		}
	}

	class GUIFeedbackManager implements GUIManager {
		public void report(String message) {
			messageArea.setText(message);
		}
	}

	class GUIEventHandler implements GUIHandler {
		public void cancel() {
			boolean modified = guiController.cancel();
			Main.this.setModified(modified);
		}

		public void pressKey(KeyEvent e) {
			if (!e.isAltDown()) {
				boolean setPoints = true;
				int keyCode = e.getKeyCode();
				if (e.isControlDown()) {
					if (keyCode == KeyEvent.VK_HOME) {
						// gui.goCtrlHome();
					} else if (keyCode == KeyEvent.VK_END) {
						// gui.goCtrlEnd();
					}
					setPoints = false;
				} else if (keyCode != KeyEvent.VK_SHIFT) {
					boolean redraw = false;
					boolean modified = false;

					if (keyCode == KeyEvent.VK_SHIFT) {
						setPoints = false;
					} else if (keyCode == KeyEvent.VK_ENTER) {
						modified = guiController.enter();
						redraw = true;
					} else if (keyCode == KeyEvent.VK_DELETE) {
						modified = guiController.delete();
					} else if (keyCode == KeyEvent.VK_BACK_SPACE) {
						modified = guiController.backSpace();
					} else if (keyCode == KeyEvent.VK_HOME) {
						guiController.goHome();
					} else if (keyCode == KeyEvent.VK_END) {
						if (e.isControlDown()) {
							guiController.goCtrlEnd();
						} else {
							guiController.goEnd();
						}
					} else if (keyCode == KeyEvent.VK_TAB) {
						if (e.isShiftDown()) {
							guiController.tabBackward();
						} else {
							guiController.tabForward();
						}
					} else if (keyCode == KeyEvent.VK_UP) {
						guiController.arrowUp();
					} else if (keyCode == KeyEvent.VK_DOWN) {
						guiController.arrowDown();
					} else if (keyCode == KeyEvent.VK_LEFT) {
						guiController.arrowLeft();
					} else if (keyCode == KeyEvent.VK_RIGHT) {
						guiController.arrowRight();
					} else if (keyCode == KeyEvent.VK_PAGE_UP) {
						guiController.pageUp();
					} else if (keyCode == KeyEvent.VK_PAGE_DOWN) {
						guiController.pageDown();
					} else if (!(e.isActionKey()
							|| keyCode == KeyEvent.VK_SHIFT || keyCode == KeyEvent.VK_ESCAPE)) {
						modified = guiController.putChar(e.getKeyChar());
					}
					if (redraw)
						guiController.getGUI().repaint();
					if (modified)
						Main.this.setModified(modified);
					if (setPoints)
						guiController.setPoints();
				}
			}
		}

		public void beginDragMouse(int x, int y) {
			guiController.mouseDragBegin(x, y);
		}

		public void dragMouse(int x, int y) {
			guiController.mouseDrag(x, y);
		}

		public void endDragMouse(int x, int y) {
			boolean modified = guiController.mouseDragEnd(x, y);
			Main.this.setModified(modified);
		}

		public void moveMouse(int x, int y) {
			guiController.mouseMove(x, y);
		}

		public void clickMouse(int x, int y) {
			guiController.mouseClick(x, y);
		}

		public void pressMouse(int x, int y) {
			guiController.mousePress(x, y);
		}
	}

	public static void main(String[] args) {
		new Main();
	}
}