/*
 * Created on Jun 12, 2006
 *
 */
package edu.ksu.cis.ndpc;

import edu.ksu.cis.viewer.TreeDrawing;

import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

/**
 * @author Brian Mulanda
 * 
 */
public class ParseTreeViewer {
	private JTextField formulaField;
	private JTextArea messageField;
	private JScrollPane pane;
	private JRadioButton rPred;
	private JRadioButton rProp;

	public static void main(String[] arg) {
		new ParseTreeViewer();
	}

	public ParseTreeViewer() {
		JFrame frame = new JFrame("PPL Parse Tree Viewer");
		frame.getContentPane().add(init());
		frame.setLocation(200, 100);
		frame.pack();
		frame.setVisible(true);
	}

	private JComponent init() {
		String formula = "AxEx(P(x0)->f(x)=f(y)) <-> (h(m)=h(c) ^ Q(x)) v (Q(m,t) ^ f(c,y,e)=f(d))";
		// String formula = "p -> q ^ r";
		JPanel panelInput = new JPanel();
		JLabel label = new JLabel("ProofFormula: ");
		formulaField = new JTextField(formula, 40);
		JButton button = new JButton("Build Parse Tree");
		panelInput.add(label);
		panelInput.add(formulaField);
		panelInput.add(button);

		JPanel panelLanguage = new JPanel();
		panelLanguage.setLayout(new FlowLayout(FlowLayout.LEFT));
		rPred = new JRadioButton("Predicate Logic");
		rProp = new JRadioButton("Propositional Logic");
		// rPred.setSelected(true);
		rProp.setSelected(true);
		panelLanguage.add(new JLabel("Language: "));
		panelLanguage.add(rPred);
		panelLanguage.add(rProp);
		ButtonGroup langGroup = new ButtonGroup();
		langGroup.add(rPred);
		langGroup.add(rProp);

		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String formula = formulaField.getText();
				parse(formula);
			}
		});

		JPanel panelMessage = new JPanel();
		messageField = new JTextArea("(Ready)", 10, 60);
		messageField.setEditable(false);
		messageField.setPreferredSize(new Dimension(300, messageField
				.getHeight()));
		panelMessage.add(messageField);

		pane = new JScrollPane();
		pane.setPreferredSize(new Dimension(200, 300));
		Box b = Box.createVerticalBox();
		b.setAlignmentY(Box.LEFT_ALIGNMENT);
		b.add(panelInput);
		b.add(panelLanguage);
		b.add(pane);
		b.add(panelMessage);
		return b;
	}

	private void parse(String formula) {
		JComponent drawing = null;
		ProofFormula pf = new ProofFormula(formula);
		try {
			LangParser parser = null;
			if (rPred.isSelected()) {
				System.out.println("Parse predicate formula: " + formula);
				parser = new PredLParser();
			} else {
				System.out.println("Parse propositional formula: " + formula);
				parser = new PropLParser();
			}
			parser.parse(pf);

			if (pf.tree != null) {
				ParseTree tree = pf.tree;
				/*
				 * ParseTree tree = (ParseTree) pf.tree.clone(); ParseTree termF
				 * = new ParseTree(new Symbol("x", LangSymbols.S_VAR));
				 * ParseTree termR = new ParseTree(new Symbol("f",
				 * LangSymbols.S_FUNC)); termR.addChild(new ParseTree(new
				 * Symbol("x", LangSymbols.S_VAR)));
				 * ProofChecker.substitute(tree, termR, termF);
				 */
				drawing = (new TreeDrawing(tree, 100)).getDrawing();
				messageField.setText("(Ready)");
			} else {
				messageField.setText("Unable to build tree");
			}
		} catch (Exception e) {
			messageField.setText(e.toString());
			// e.printStackTrace();
		} finally {
			pane.setViewportView(drawing);
		}
	}

	public void viewTree(String formula, ParseTree tree) {
		JComponent drawing = null;
		if (tree != null) {
			formulaField.setText(formula);
			drawing = (new TreeDrawing(tree, 100)).getDrawing();
			messageField.setText("(Ready)");
		} else {
			messageField.setText("Unable to build tree");
		}
		pane.setViewportView(drawing);
	}
}
