package edu.ksu.cis.ndpc;

/**
 * @author Brian Mulanda
 *
 */

import edu.ksu.cis.viewer.EmptyTreeException;
import edu.ksu.cis.viewer.TreeInterface;
import java.util.LinkedList;

public class ProofTree implements TreeInterface {
	private ProofTreeNode root;
	private LinkedList children;
	private ProofTree parent;
	private boolean allowChildren;

	public ProofTree(ProofTreeNode root) {
		this.root = root;
		this.parent = null;
		this.allowChildren = root.allowsChildren();
		if (root != null && root.allowsChildren()) {
			children = new LinkedList();
		}
	}

	public ProofTree getParent() {
		return parent;
	}

	public void addChild(ProofTree child) {
		addChild(child, false);
	}

	public void addChild(ProofTree child, boolean reverse) {
		if (allowChildren && (child != null)) {
			if (reverse) {
				this.children.addFirst(child);
			} else {
				this.children.add(child);
			}
			child.parent = this;
		}
	}

	public ProofTreeNode getRootNode() {
		return root;
	}

	public ProofTree getFirstSubtree() {
		if (allowChildren && children.size() > 0) {
			return (ProofTree) children.getFirst();
		} else {
			return null;
		}
	}

	public ProofTree getLastSubtree() {
		if (allowChildren && children.size() > 0) {
			return (ProofTree) children.getLast();
		} else {
			return null;
		}
	}

	public ProofTree[] getSubtrees() {
		ProofTree[] childArray = null;
		if (allowChildren) {
			childArray = new ProofTree[children.size()];
			for (int i = 0; i < children.size(); ++i) {
				childArray[i] = (ProofTree) children.get(i);
			}
		}
		return childArray;
	}

	/*
	 * Begin: Methods due to TreeInterface getRoot(); getChildren(); isEmpty();
	 */

	public Object getRoot() throws EmptyTreeException {
		return root;
	}

	public TreeInterface[] getChildren() throws EmptyTreeException {
		TreeInterface[] childArray = null;
		if (allowChildren) {
			childArray = new TreeInterface[children.size()];
			for (int i = 0; i < children.size(); ++i) {
				childArray[i] = (TreeInterface) children.get(i);
			}
		}
		return childArray;
	}

	public boolean isEmpty() {
		return false;
	}

	/*
	 * End: Methods due to TreeInterface
	 */
}