package edu.ksu.cis.ndpc;

/**
 * @author Brian Mulanda
 *
 */

import java.util.LinkedList;

public class SymbolTokenizer {
	private Symbol symbol;
	private Symbol nextSymbol;
	private Tokenizer tokenizer;
	private LinkedList buffer;
	private LangSymbols symbols;

	public SymbolTokenizer(String formula) {
		this(formula, null);
	}

	public SymbolTokenizer(String formula, LangSymbols symbols) {
		this.tokenizer = new Tokenizer(formula, symbols.getSpecialSymbols());
		this.buffer = new LinkedList();
		this.symbols = symbols;
	}

	public boolean nextSymbol() {
		return nextSymbol(false);
	}

	public boolean nextSymbol(boolean query) {
		boolean result = false;
		if (query) {
			nextSymbol = buildSymbol();
			if (nextSymbol != null) {
				buffer.addLast(nextSymbol);
				result = true;
			} else {
				result = false;
			}
		} else {
			if (buffer.size() > 0) {
				symbol = (Symbol) buffer.removeFirst();
			} else {
				symbol = buildSymbol();
			}
			result = (symbol != null);
		}
		return (result);
	}

	public Symbol getSymbol() {
		return getSymbol(false);
	}

	public Symbol getSymbol(boolean query) {
		Symbol symb = null;
		if (query) {
			if (buffer.size() > 0) {
				symb = (Symbol) buffer.getFirst();
			}
		} else {
			symb = symbol;
		}
		return symb;
	}

	private Symbol buildSymbol() {
		Symbol symb = null;

		Token token = buildToken(null);
		if (token != null) {
			int type = symbols.identify(token.getValue());
			symb = new Symbol(token.getValue(), type, token.getPosition(),
					token.getLength());
		}

		return symb;
	}

	private Token buildToken(Token prevToken) {
		Token nextToken = null;
		if (tokenizer.nextToken()) {
			Token token = tokenizer.getToken();
			boolean build = false;

			if (token.getValue().trim().equals("<")
					|| token.getValue().trim().equals("-")
					|| token.getValue().trim().equals("|")) {
				build = true;
			} else if (token.getValue().trim().equals("_") && prevToken == null) {
				build = true;
			}
			if (build) {
				Token tempToken = buildToken(token);
				if (tempToken != null) {
					token = new Token(token.getValue() + tempToken.getValue(),
							token.getPosition(), token.getLength()
									+ tempToken.getLength());
				}
			}
			nextToken = new Token(token.getValue(), token.getPosition(), token
					.getLength());
		}
		return nextToken;
	}

	public void pushBack() {
		buffer.addFirst(symbol);
	}
}