package edu.ksu.cis.ndpc.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;

public class BoxHandler {
	public static final int HANDLE_NONE = -1;
	public static final int HANDLE_TLEFT = 0;
	public static final int HANDLE_TCENTER = 1;
	public static final int HANDLE_TRIGHT = 2;
	public static final int HANDLE_CLEFT = 3;
	public static final int HANDLE_CRIGHT = 4;
	public static final int HANDLE_BLEFT = 5;
	public static final int HANDLE_BCENTER = 6;
	public static final int HANDLE_BRIGHT = 7;
	public static final int HANDLE_TOP = 8;
	public static final int HANDLE_BOTTOM = 9;
	public static final int HANDLE_CENTER = 10;

	private static int spacing = 0;
	private ArrayList boxList;
	private GUIBox rootbox;
	private GUIBox selected;
	private FillHandle handle;

	public BoxHandler() {
		boxList = new ArrayList();
		selected = null;
		rootbox = new GUIBox(Integer.MIN_VALUE / 2, Integer.MAX_VALUE / 2);
		handle = new FillHandle();
	}

	public static int getSpacing() {
		return BoxHandler.spacing;
	}

	public static void setSpacing(int value) {
		BoxHandler.spacing = value;
	}

	public int height() {
		return rootbox.height();
	}

	public int boxCount() {
		return boxList.size();
	}

	public FillHandle getHandle() {
		return handle;
	}

	public GUIBox get(int index) {
		if (index >= 0 && index < boxList.size()) {
			return (GUIBox) boxList.get(index);
		} else {
			return null;
		}
	}

	/*
	 * public int[][] boxesToArray() { int[][] boxes = null; GUIBox box = null;
	 * int count = boxList.size(); if (count > 0) { boxes = new int[count][2];
	 * for (int i = 0; i < count; ++i) { box = (GUIBox)boxList.get(i);
	 * boxes[i][0] = box.startRow(); boxes[i][1] = box.endRow(); } } return
	 * boxes; }
	 */

	public boolean containsBox(int startRow, int endRow) {
		return rootbox.contains(startRow, endRow);
	}

	public GUIBox addBox(int startRow, int endRow) {
		return addBox(startRow, endRow, null);
	}

	public GUIBox addBox(int startRow, int endRow, Boundary bounds) {
		GUIBox b = null;
		if (startRow < endRow) {
			b = new GUIBox(startRow, endRow);
		} else {
			b = new GUIBox(endRow, startRow);
		}
		rootbox.add(b);
		boxList.add(b);
		if (bounds != null)
			b.setBounds(bounds);
		return b;
	}

	public GUIBox addBox(GUIBox box) {
		if (box != null) {
			rootbox.add(box);
			boxList.add(box);
		}
		return box;
	}

	public void removeAllBoxes() {
		selected = null;
		rootbox = new GUIBox(Integer.MIN_VALUE / 2, Integer.MAX_VALUE / 2);
		handle = new FillHandle();
		boxList.clear();
	}

	public void drawBoxes(Graphics g) {
		GUIBox b = null;
		for (int i = 0; i < boxList.size(); ++i) {
			b = (GUIBox) boxList.get(i);
			this.drawBox(g, b);
		}
		if (selected != null)
			highLightSelectedBox(g);
	}

	public void highLightSelectedBox(Graphics g) {
		highLightBox(g, selected);
	}

	public void highLightBox(Graphics g, GUIBox box) {
		if (box != null) {
			Boundary b = box.getBounds();
			Color c = g.getColor();
			g.setColor(Color.RED);
			g.drawRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
			g.drawRect(b.x1 - 1, b.y1 - 1, b.x2 - b.x1 + 2, b.y2 - b.y1 + 2);

			// Top left
			b = handle.topLeft;
			// System.out.println("fill b: " + b);
			g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
			// Top right
			b = handle.topRight;
			g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
			// Bottom left
			b = handle.bottomLeft;
			g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
			// Bottom right
			b = handle.bottomRight;
			g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
			// Top middle
			b = handle.topCenter;
			g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
			// Bottom middle
			b = handle.bottomCenter;
			g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
			g.setColor(c);
		}
	}

	public void drawBox(Graphics g, GUIBox box) {
		// System.out.println("drawBox: " + box + ", " + box.getBounds());
		Boundary b = box.getBounds();
		g.drawRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
	}

	public void drawBox(Graphics g, Boundary b) {
		if (b.y1 < b.y2) {
			g.drawRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		} else if (b.y1 > b.y2) {
			g.drawRect(b.x1, b.y2, b.x2 - b.x1, b.y1 - b.y2);
		} else {
			g.drawLine(b.x1, b.y1, b.x2, b.y2);
		}
	}

	public void clearBox(Graphics g, Boundary b) {
		Color c = g.getColor();
		g.setColor(Color.WHITE);
		if (b.y1 < b.y2) {
			g.drawRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		} else if (b.y1 > b.y2) {
			g.drawRect(b.x1, b.y2, b.x2 - b.x1, b.y1 - b.y2);
		} else {
			g.drawLine(b.x1, b.y1, b.x2, b.y2);
		}
		g.setColor(c);
	}

	public void clearHighlightedBox(Graphics g) {
		Boundary b = selected.getBounds();
		Color c = g.getColor();
		g.setColor(Color.WHITE);
		g.drawRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		g.drawRect(b.x1 - 1, b.y1 - 1, b.x2 - b.x1 + 2, b.y2 - b.y1 + 2);

		// Top left
		b = handle.topLeft;
		// System.out.println("fill b: " + b);
		g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		// Top right
		b = handle.topRight;
		g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		// Bottom left
		b = handle.bottomLeft;
		g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		// Bottom right
		b = handle.bottomRight;
		g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		// Top middle
		b = handle.topCenter;
		g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		// Bottom middle
		b = handle.bottomCenter;
		g.fillRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		g.setColor(c);
	}

	public GUIBox closestBox(Point p) {
		return closestBox(p.x, p.y);
	}

	public GUIBox closestBox(int x, int y) {
		int index = -1;
		GUIBox box = null;
		Boundary b = null;
		// /*
		if (selected != null) {
			b = selected.getBounds();
			if ((b.x1 - x >= -2 && b.x1 - x <= 2 && y >= b.y1 && y <= b.y2)
					|| (b.x2 - x >= -2 && b.x2 - x <= 2 && y >= b.y1 && y <= b.y2)
					|| (b.y1 - y >= -2 && b.y1 - y <= 2 && x >= b.x1 && x <= b.x2)
					|| (b.y2 - y >= -2 && b.y2 - y <= 2 && x >= b.x1 && x <= b.x2)) {

				box = selected;
			}
		}
		// */
		if (box == null) {
			for (int i = 0; i < boxList.size(); ++i) {
				box = (GUIBox) boxList.get(i);
				b = box.getBounds();
				if ((b.x1 - x >= -2 && b.x1 - x <= 2 && y >= b.y1 && y <= b.y2)
						|| (b.x2 - x >= -2 && b.x2 - x <= 2 && y >= b.y1 && y <= b.y2)
						|| (b.y1 - y >= -2 && b.y1 - y <= 2 && x >= b.x1 && x <= b.x2)
						|| (b.y2 - y >= -2 && b.y2 - y <= 2 && x >= b.x1 && x <= b.x2)) {

					index = i;
					i = boxList.size();
				}
			}
			if (index == -1) {
				box = null;
			}
		}

		return box;
	}

	public void selectBox(GUIBox box) {
		if (box != null) {
			selected = box;
			handle = computeHandle(selected.getBounds());
			/*
			 * for ( int i = 0; i < boxList.size(); ++i ) { GUIBox b =
			 * (GUIBox)boxList.get(i); if (b.equals(box)) { boxList.remove(i);
			 * boxList.add(0, box); selected = box; handle =
			 * computeHandle(selected.getBounds()); i = boxList.size(); } }
			 */
		}
	}

	public void reselectBox() {
		if (selected != null) {
			handle = computeHandle(selected.getBounds());
		}
	}

	public GUIBox getSelectedBox() {
		return selected;
	}

	public boolean canAddBox(int startRow, int endRow) {
		boolean canAdd = true;
		if (startRow == endRow) {
			canAdd = false;
		} else {
			if (startRow > endRow) {
				int temp = startRow;
				startRow = endRow;
				endRow = temp;
			}

			GUIBox b = new GUIBox(startRow, endRow);
			if (rootbox.contains(b)) {
				canAdd = false;
			} else {
				// Check for overlap as well as duplicate box
				GUIBox box = null;
				int size = boxList.size();
				for (int i = 0; i < size; ++i) {
					box = (GUIBox) boxList.get(i);
					if (box.strictOverlaps(b) || box.equals(b)) {
						canAdd = false;
						i = size;
					}
				}
			}
			if (canAdd) {
				// System.out.println("can add:"+b);
			} else {
				// System.out.println("cannot add:"+b);
			}
		}
		return canAdd;
	}

	public void removeSelectedBox() {
		for (int i = 0; i < boxList.size(); ++i) {
			GUIBox b = (GUIBox) boxList.get(i);
			if (b.equals(selected)) {
				boxList.remove(i);
				rootbox.remove(b);
				i = boxList.size();
			}
		}
		clearSelection();
	}

	public void clearSelection() {
		// selected = null;
		selected = null;
	}

	public FillHandle computeHandle(Boundary b) {
		if (b.y1 <= b.y2) {
			return computeHandle(b.x1, b.x2, b.y1, b.y2);
		} else {
			return computeHandle(b.x1, b.x2, b.y2, b.y1);
		}
	}

	public FillHandle computeHandle(int x1, int x2, int y1, int y2) {
		int w = 3;
		FillHandle h = new FillHandle();

		h.topLeft = new Boundary(x1 - w, y1 - w, x1 + w, y1 + w);
		h.topCenter = new Boundary(x1 + (x2 - x1) / 2 - w, y1 - w, x1
				+ (x2 - x1) / 2 + w, y1 + w);
		h.topRight = new Boundary(x2 - w, y1 - w, x2 + w, y1 + w);

		h.bottomLeft = new Boundary(x1 - w, y2 - w, x1 + w, y2 + w);
		h.bottomCenter = new Boundary(x1 + (x2 - x1) / 2 - w, y2 - w, x1
				+ (x2 - x1) / 2 + w, y2 + w);
		h.bottomRight = new Boundary(x2 - w, y2 - w, x2 + w, y2 + w);

		return h;
	}

	public void insertRow(int row) {
		rootbox.insertRow(row);
	}

	public void appendRow(int row) {
		rootbox.insertRow(row);
	}

	public void removeRow(int row) {
		rootbox.removeRow(row);
		rebuildList();
	}

	private void rebuildList() {
		boxList.clear();
		addToList(rootbox.getChildren());
	}

	private void addToList(Object[] boxes) {
		for (int i = 0; i < boxes.length; ++i) {
			GUIBox b = (GUIBox) boxes[i];
			boxList.add(b);
			addToList(b.getChildren());
		}
	}

	public int getHandleType(FillHandle handle, Point p) {
		return getHandleType(handle, p.x, p.y);
	}

	public int getHandleType(FillHandle handle, int x, int y) {
		int h = HANDLE_NONE;
		if (handle.topLeft.containsPoint(x, y)) {
			h = HANDLE_TLEFT;
		} else if (handle.topCenter.containsPoint(x, y)) {
			h = HANDLE_TCENTER;
		} else if (handle.topRight.containsPoint(x, y)) {
			h = HANDLE_TRIGHT;
		} else if (handle.centerLeft.containsPoint(x, y)) {
			h = HANDLE_CLEFT;
		} else if (handle.centerRight.containsPoint(x, y)) {
			h = HANDLE_CRIGHT;
		} else if (handle.bottomLeft.containsPoint(x, y)) {
			h = HANDLE_BLEFT;
		} else if (handle.bottomCenter.containsPoint(x, y)) {
			h = HANDLE_BCENTER;
		} else if (handle.bottomRight.containsPoint(x, y)) {
			h = HANDLE_BRIGHT;
		}
		return h;
	}

	public int getHandlePos(FillHandle handle, int x, int y) {
		int h = HANDLE_NONE;
		if (handle.topLeft.containsPoint(x, y)
				|| handle.topCenter.containsPoint(x, y)
				|| handle.topRight.containsPoint(x, y)) {
			h = HANDLE_TOP;
		} else if (handle.centerLeft.containsPoint(x, y)
				|| handle.centerRight.containsPoint(x, y)) {
			h = HANDLE_CENTER;
		} else if (handle.bottomLeft.containsPoint(x, y)
				|| handle.bottomCenter.containsPoint(x, y)
				|| handle.bottomRight.containsPoint(x, y)) {
			h = HANDLE_BOTTOM;
		}
		return h;
	}

	public void print() {
		rootbox.print(1);
	}
}