package edu.ksu.cis.ndpc.gui;

import javax.swing.JPanel;

import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.Scrollable;
import javax.swing.JComponent;

import edu.ksu.cis.ndpc.LineRefParser;
import edu.ksu.cis.ndpc.LineRefParserException;
import edu.ksu.cis.ndpc.LineReference;
import edu.ksu.cis.ndpc.Range;

public abstract class GUIController {
	protected static JPanel gui;
	private static boolean initialized = false;
	private static GUIManager guimgr;

	private static final int CONTROLLER_PRED = 0;
	private static final int CONTROLLER_PROP = 1;

	private static final Cursor defaultCursor = new Cursor(
			Cursor.DEFAULT_CURSOR);
	private static final Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
	private static final Cursor TC_CURSOR = new Cursor(Cursor.N_RESIZE_CURSOR);
	private static final Cursor CL_CURSOR = new Cursor(Cursor.W_RESIZE_CURSOR);
	private static final Cursor CR_CURSOR = new Cursor(Cursor.E_RESIZE_CURSOR);
	private static final Cursor BC_CURSOR = new Cursor(Cursor.S_RESIZE_CURSOR);
	// private static final Cursor MOVE_CURSOR = new Cursor(Cursor.MOVE_CURSOR);

	private static final int STATE_DEFAULT = 0;
	private static final int STATE_BOX_SELECT = 10;

	private static final int MIN_VIEW_ROWS = 9;

	protected static int rowSpacing;
	protected static int colSpacing;
	protected static int boxPixels;
	protected static int boxHPixels;
	protected static int cellPadding;
	protected static int cellIncrease;

	protected static int state;

	protected static boolean boxSelected;
	protected static boolean drawingBox;
	protected static boolean resizingBox;
	protected static boolean movingBox;

	protected static Point point;
	protected static Margin margin;
	protected static TextCursor cursor;
	protected static TextCursorManager tcm;
	protected static Drawer drawer;

	protected static BoxHandler boxMan;
	protected static LineRefParser lineParser;

	private Boundary newBox;
	private Boundary resizeBox;
	private Boundary moveBox;

	private static int drawBoxOffset;
	private int boxStartRow;
	private int boxEndRow;
	private int moveY;

	private static GUIBox selectedBox;

	protected GUIContent content;

	private static int cursorVOffset;
	private static int cursorHOffset;

	private static GUIController predLController = null;
	private static GUIController propLController = null;
	private static GUIController controller = null;

	public static void addMouseListener(MouseListener listener) {
		GUIController.gui.addMouseListener(listener);
	}

	public static void addMouseMotionListener(MouseMotionListener listener) {
		GUIController.gui.addMouseMotionListener(listener);
	}

	public static void addKeyListener(KeyListener listener) {
		GUIController.gui.addKeyListener(listener);
	}

	public static void setGUIManager(GUIManager guimgr) {
		GUIController.guimgr = guimgr;
	}

	private static void resizeGUI() {
		resizeGUIWidth(false);
		resizeGUIHeight(false);
		gui.revalidate();
	}

	private static void resizeGUIHeight(boolean revalidate) {
		int rows = controller.content.getRowCount();
		if (rows < MIN_VIEW_ROWS)
			rows = MIN_VIEW_ROWS;
		int height = margin.sumVertical() + (rows * getRowHeight());

		gui
				.setPreferredSize(new Dimension(gui.getPreferredSize().width,
						height));
		if (revalidate)
			gui.revalidate();
	}

	private static void resizeGUIWidth(boolean revalidate) {
		int rowsize = controller.getRowsize();
		int width = margin.sumHorizontal() + (rowsize * drawer.getCharWidth())
				+ controller.getColSpacing(0, controller.getLinerefColumn());

		gui
				.setPreferredSize(new Dimension(width,
						gui.getPreferredSize().height));
		controller.resizeColumns();
		if (revalidate)
			gui.revalidate();
	}

	public void setFontsize(int fontsize) {
		if (controller != null) {
			Font f = gui.getFont().deriveFont((float) fontsize);
			gui.setFont(f);
			FontMetrics fm = gui.getFontMetrics(f);
			drawer = new Drawer(fm.charWidth('A'), fm.getMaxAscent());
			Row.setAscent(fm.getMaxAscent());
			drawBoxOffset = (getRowHeight() * 2) / 3;
			resizeGUI();
			setPoints();
			gui.repaint();
		}
	}

	public static GUIController setProof(RawProof proof) {
		if (proof != null && proof.getType() != RawProof.PROOF_UNKNOWN) {
			if (!tcm.isPaused())
				tcm.pause();

			switch (proof.getType()) {
			case RawProof.PROOF_PRED:
				controller = getController(CONTROLLER_PRED);
				break;
			case RawProof.PROOF_PROP:
				controller = getController(CONTROLLER_PROP);
				break;
			default:
				controller = null;
				break;
			}
		} else {
			controller = null;
		}

		if (controller != null) {
			controller.setupProof(proof);

			boxMan.removeAllBoxes();
			for (int i = 0; i < proof.boxCount(); ++i) {
				RawBox box = proof.getBox(i);
				boxMan.addBox(box.startRow(), box.endRow() + 1);
			}

			resizeGUI();
			controller.goToCol(controller.getFormulaColumn());
			controller.goToFirstRow();
			controller.goHome();
			controller.setPoints();

			tcm.resume();
		}

		return controller;
	}

	protected abstract void setupProof(RawProof proof);

	protected abstract int getRowsize();

	protected abstract int getRowsize(int col);

	protected abstract void setRowsize(int col, int size);

	protected abstract void resetRowsize(int col);

	public static void init() {
		init(16);
	}

	public static void init(int fontsize) {
		if (!initialized && fontsize >= 8) {
			Font font = new Font("Monospaced", 0, fontsize);

			gui = new GUI();
			gui.setFocusable(true);
			gui.setFocusTraversalKeysEnabled(false);
			gui.setFont(font);

			boxMan = new BoxHandler();
			tcm = new TextCursorManager();
			lineParser = new LineRefParser();

			margin = new Margin(5, 5, 5, 5); // top, left, right, bottom
			point = new Point(margin.left, margin.top);
			cursor = new TextCursor();

			BoxHandler.setSpacing(3);

			colSpacing = 5;
			rowSpacing = 2;
			cellPadding = 5;
			cellIncrease = cellPadding + 3;

			FontMetrics fm = gui.getFontMetrics(font);
			drawer = new Drawer(fm.charWidth('A'), fm.getMaxAscent());

			Row.setLPad(1);
			Row.setRPad(1);
			Row.setTPad(2);
			Row.setBPad(3);
			Row.setAscent(fm.getMaxAscent());

			cursorVOffset = (Row.getHeight() * 3) / 2;
			cursorHOffset = 200;

			boxPixels = 1;
			boxHPixels = 1;

			state = STATE_DEFAULT;
			boxSelected = false;
			drawingBox = false;
			resizingBox = false;
			movingBox = false;

			drawBoxOffset = (getRowHeight() * 2) / 3;
			tcm.start();
			initialized = true;
		}
	}

	private static GUIController getController(int type) {
		GUIController c = null;
		switch (type) {
		case CONTROLLER_PRED:
			if (predLController == null) {
				predLController = new PredGUIController();
				initController(predLController);
			}
			c = predLController;
			break;

		case CONTROLLER_PROP:
			if (propLController == null) {
				propLController = new PropGUIController();
				initController(propLController);
			}
			c = propLController;
			break;

		default:
			break;
		}
		return c;
	}

	private static void initController(GUIController c) {
		cursor.col = c.getFormulaColumn();
		c.setupColumns();
	}

	static class GUI extends JPanel implements Scrollable {
		static final long serialVersionUID = 1;

		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			redraw(g);
		}

		public Dimension getPreferredScrollableViewportSize() {
			return getPreferredSize();
		}

		public int getScrollableBlockIncrement(Rectangle visibleRect,
				int orientation, int direction) {
			return 0;
		}

		public boolean getScrollableTracksViewportHeight() {
			return false;
		}

		public boolean getScrollableTracksViewportWidth() {
			return false;
		}

		public int getScrollableUnitIncrement(Rectangle visibleRect,
				int orientation, int direction) {

			return 0;
		}
	}

	public JComponent getGUI() {
		return gui;
	}

	public BufferedImage getImage() {
		BufferedImage image = new BufferedImage(gui.getWidth(),
				gui.getHeight(), BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 = image.createGraphics();
		gui.paint(g2);
		g2.dispose();

		return image;
	}

	protected abstract int getVariableColumn();

	protected abstract int getFormulaColumn();

	protected abstract int getLinerefColumn();

	protected void report(String message) {
		if (guimgr != null) {
			guimgr.report(message);
		}
	}

	protected abstract void setupColumns();

	private static void redraw(Graphics g) {
		// System.out.println("Redraw:begin: " + cursor);
		int cursor_col = cursor.col;
		int cursor_row = cursor.row;
		int cursor_off = cursor.offset;

		Color c = g.getColor();
		int width = 0;
		int height = 0;
		int x = margin.left;
		int y = margin.top;
		cursor.offset = 0;
		height = getColHeight();
		for (int i = 0; i < controller.content.getColumnCount(); ++i) {
			width = controller.getColWidth(i);
			g.setColor(Color.WHITE);
			g.fillRect(x, y, width, height);
			g.setColor(c);
			x += (width + controller.getColSpacing(i, i + 1));

			Col col = controller.content.getCol(i);
			for (int j = 0; j < col.rowCount(); ++j) {
				Row row = col.getRow(j);
				if (row.length() > 0) {
					cursor.col = i;
					cursor.row = j;
					char[] data = row.getData();
					Point p = cursorToPoint(cursor);
					p.y += boxPixels + boxHPixels + Row.getTPad()
							+ Row.getAscent();
					drawer.draw(g, data, p.x, p.y, true);
				}
			}
		}
		drawBoxes(g);
		cursor.col = cursor_col;
		cursor.row = cursor_row;
		cursor.offset = cursor_off;
		// System.out.println("Redraw:end: " + cursor);
	}

	public void pauseCursor() {
		tcm.pause();
	}

	public void resumeCursor() {
		tcm.resume();
	}

	public void tabBackward() {
		if (isFirstCol(cursor.col)) {
			if (!isFirstRow(cursor.row)) {
				goToLastCol();
				goToPrevRow();
				goEnd();
			}
		} else {
			goToPrevCol();
			goEnd();
		}
	}

	public void tabForward() {
		if (isLastCol(cursor.col)) {
			if (!isLastRow(cursor.row)) {
				goToNextRow();
				goToFirstCol();
				goHome();
			}
		} else {
			goToNextCol();
			goHome();
		}
		// System.out.println("tabforward: " + cursor);
	}

	public void arrowDown() {
		if (!isLastRow(cursor.row)) {
			goToNextRow();
			if (!cursorOk(cursor)) {
				cursor = fixCursor(cursor);
			}
		}
	}

	public void arrowUp() {
		if (!isFirstRow(cursor.row)) {
			goToPrevRow();
			if (!cursorOk(cursor)) {
				cursor = fixCursor(cursor);
			}
		}
	}

	public void arrowLeft() {
		if (cursor.offset > 0) {
			--(cursor.offset);
		} else {
			tabBackward();
		}
	}

	public void arrowRight() {
		if (cursor.offset < content.getCol(cursor.col).getRow(cursor.row)
				.length()) {
			++(cursor.offset);
		} else {
			tabForward();
		}
	}

	public void goExtremeLeft() {
		cursor.col = getFormulaColumn();

	}

	public void goCtrlEnd() {
		cursor.col = getLinerefColumn();
		cursor.row = content.getRowCount() - 1;
		cursor.offset = content.getCol(cursor.col).getRow(cursor.row).length();
	}

	public void goCtrlHome() {
		cursor.col = getFormulaColumn();
		cursor.row = 0;
		cursor.offset = 0;
	}

	public void goHome() {
		cursor.offset = 0;
	}

	public void goEnd() {
		cursor.offset = content.getCol(cursor.col).getRow(cursor.row).length();
	}

	public void pageUp() {
		Rectangle aRect = gui.getVisibleRect();
		cursor = pointToCursor(point.x, point.y - aRect.height);
		cursor = fixCursor(cursor);
	}

	public void pageDown() {
		Rectangle aRect = gui.getVisibleRect();
		cursor = pointToCursor(point.x, point.y + aRect.height);
		cursor = fixCursor(cursor);
	}

	public boolean delete() {
		boolean modified = false;
		if (state == STATE_DEFAULT) {
			Row row = content.getCol(cursor.col).getRow(cursor.row);
			int rowlength = row.length();
			if (cursor.offset < rowlength) {
				row.deleteCharAt(cursor.offset);

				int rowsize = getRowsize(cursor.col);
				if (rowlength < rowsize - cellIncrease) {
					if (rowlength + 1 == rowsize - cellIncrease) {
						resetRowsize(cursor.col);
						resizeGUIWidth(true);
					}
				}
				gui.repaint();
				modified = true;
			} else {
				if (isLastCol(cursor.col)
						&& content.getRowLength(cursor.row, true) == 0) {
					if (!isLastRow(cursor.row)) {
						removeRow(cursor.row);
						goToFirstCol();
						goHome();
						gui.repaint();
						modified = true;
					}
				} else {
					tabForward();
				}
			}
		} else if (state == STATE_BOX_SELECT) {
			if (boxSelected) {
				boxMan.removeSelectedBox();
				boxSelected = false;
				state = STATE_DEFAULT;
				resizeColumns();
				gui.repaint();
				modified = true;
			}
		}
		return modified;
	}

	public boolean backSpace() {
		boolean modified = false;
		if (cursor.offset > 0) {
			--(cursor.offset);
			Row row = content.getCol(cursor.col).getRow(cursor.row);
			row.deleteCharAt(cursor.offset);

			int rowsize = getRowsize(cursor.col);
			int rowlength = row.length();

			if (rowlength < rowsize - cellIncrease) {
				if (rowlength + 1 == rowsize - cellIncrease) {
					resetRowsize(cursor.col);
					resizeGUIWidth(true);
				}
			}
			gui.repaint();
			modified = true;
		} else {
			if (isFirstCol(cursor.col)
					&& content.getRowLength(cursor.row, true) == 0) {
				if (!isFirstRow(cursor.row)) {
					removeRow(cursor.row);
					goToPrevRow();
					goToLastCol();
					goEnd();
					gui.repaint();
					modified = true;
				}
			} else {
				tabBackward();
			}
		}
		return modified;
	}

	public boolean putChar(char c) {
		boolean modified = true;

		Col col = content.getCol(cursor.col);
		Row row = col.getRow(cursor.row);
		if (cursor.offset < row.length()) {
			row.insert(cursor.offset, c);
		} else {
			row.append(c);
		}
		++(cursor.offset);

		int rowsize = getRowsize(cursor.col);
		if (row.length() > rowsize - cellPadding) {
			int newRowsize = rowsize + cellIncrease;
			setRowsize(cursor.col, newRowsize);
			resizeGUI();
		}
		gui.repaint();

		return modified;
	}

	public boolean enter() {
		boolean modified = true;
		Col col = content.getCol(cursor.col);

		int nextRow = cursor.row + 1;
		Row currRow = col.getRow(cursor.row);

		insertRow(nextRow);
		if (!(isLastCol(cursor.col) && cursor.offset == currRow.length())) {
			for (int i = cursor.col + 1; i < content.getColumnCount(); ++i) {
				Col tempCol = content.getCol(i);
				tempCol.getRow(nextRow).append(
						tempCol.getRow(cursor.row).getData());
				tempCol.getRow(cursor.row).deleteAll();
			}
			col.getRow(nextRow).append(currRow.subchars(cursor.offset));
			currRow.deleteRest(cursor.offset);
		}

		int updateRow = nextRow;
		boolean blankRow = (content.getRowLength(cursor.row, true) == 0);
		int textBefCursor = 0;
		for (int i = 1; i < cursor.col && textBefCursor == 0; ++i) {
			textBefCursor += content.getCol(i).getRow(cursor.row).length(true);
		}
		if (textBefCursor == 0) {
			textBefCursor += currRow.substring(0, cursor.offset).trim()
					.length();
		}

		if (!blankRow) {
			if (textBefCursor == 0) {
				updateRow = cursor.row;
			} else { /* skip */
			}
		} else if (cursor.col <= getFormulaColumn()) {
			updateRow = cursor.row;
		}
		updateLineRefs(updateRow, content.getRowCount() - 1, 1);

		recalcBoxBounds();
		goToFormulaCol();
		goToNextRow();
		goHome();

		return modified;
	}

	public boolean cancel() {
		boolean modified = false;
		if (drawingBox) {
			newBox = null;
			drawingBox = false;
		} else if (resizingBox) {
			resizingBox = false;
			resizeBox = null;
		} else if (movingBox) {
			movingBox = false;
			moveBox = null;
		}
		gui.repaint();
		return modified;
	}

	public boolean drawBox(int startRow, int endRow) {
		return drawBox(startRow, endRow, false);
	}

	public boolean drawBox(int startRow, int endRow, boolean repaint) {
		boolean draw = (addBox(startRow, endRow) != null);
		if (draw && repaint) {
			gui.repaint();
		}
		return draw;
	}

	public void mouseClick(int x, int y) {
		GUIBox b = boxMan.closestBox(x, y);
		if (b == null) {
			TextCursor clickCursor = pointToCursor(x, y);
			if (cursorOk(clickCursor)) {
				cursor = clickCursor;
			} else {
				cursor = fixCursor(clickCursor);
			}
			setPoints();
		}
	}

	public void mouseDrag(int x, int y) {
		if (drawingBox) {
			newBox(x, y, true);
		} else if (resizingBox) {
			resizeBox(x, y, true);
		} else if (movingBox) {
			moveBox(x, y, true);
		}

		Rectangle aRect = gui.getVisibleRect();
		int aRectY = aRect.y + aRect.height;
		boolean scroll = false;
		if (y > aRectY - Row.getHeight()) {
			aRect.translate(0, y - aRectY + Row.getHeight());
			scroll = true;
		} else if (y < aRect.y + Row.getHeight()) {
			aRect.translate(0, y - (aRect.y + Row.getHeight()));
			scroll = true;
		}

		if (scroll)
			gui.scrollRectToVisible(aRect);
	}

	public void mouseDragBegin(int x, int y) {
		// System.out.println("Begin drag mouse: " + new Point(x,y));
		GUIBox b = boxMan.closestBox(x, y);
		if (b == null) {
			drawingBox = true;
			boxStartRow = pointToRow(x, y);
			if (Math.abs(rowTop(boxStartRow) - y) >= getRowHeight() / 2) {
				++boxStartRow;
			}
			int endRow = boxStartRow;
			newBox = calcBoxBounds(boxStartRow, endRow, 0);
			boxMan.drawBox(gui.getGraphics(), newBox);

		} else if (b == boxMan.getSelectedBox()) {
			int handlePos = boxMan.getHandlePos(boxMan.getHandle(), x, y);
			// System.out.println("Handle Pos: " + handlePos);
			selectedBox = b;
			if (handlePos == BoxHandler.HANDLE_TOP
					|| handlePos == BoxHandler.HANDLE_BOTTOM) {

				resizeBox = b.getBounds();
				if (handlePos == BoxHandler.HANDLE_TOP) {
					boxStartRow = b.endRow();
					int tempY = resizeBox.y1;
					resizeBox.y1 = resizeBox.y2;
					resizeBox.y2 = tempY;
				} else {
					boxStartRow = b.startRow();
				}
				resizingBox = true;
			} else {
				/*
				 * boxStartRow = b.startRow(); boxEndRow = b.endRow(); movingBox
				 * = true; moveBox = b.getBounds(); moveY = y;
				 * //System.out.println("Moving:" + b);
				 * gui.setCursor(MOVE_CURSOR);
				 */
			}
		}
	}

	public boolean mouseDragEnd(int x, int y) {
		// System.out.println("End drag mouse");
		boolean modified = false;
		if (drawingBox) {
			modified = newBox(x, y, false);
		} else if (resizingBox) {
			modified = resizeBox(x, y, false);
		} else if (movingBox) {
			modified = moveBox(x, y, false);
		}
		return modified;
	}

	public void mousePress(int x, int y) {
		gui.requestFocusInWindow();
		GUIBox box = boxMan.closestBox(x, y);
		if (box != null) {
			boolean redraw = boxSelected;
			state = STATE_BOX_SELECT;
			boxMan.selectBox(box);
			boxSelected = true;
			tcm.pause();
			if (redraw) {
				gui.repaint();
			} else {
				boxMan.highLightSelectedBox(gui.getGraphics());
			}
			tcm.resume();
		} else {
			state = STATE_DEFAULT;
			if (boxSelected) {
				boxSelected = false;
				boxMan.clearSelection();
				gui.repaint();
			}
		}
	}

	public void mouseMove(int x, int y) {
		GUIBox b = boxMan.closestBox(x, y);
		if (b != null) {
			if (boxMan.getSelectedBox() == b) {
				int handle = boxMan.getHandleType(boxMan.getHandle(), x, y);
				switch (handle) {
				case BoxHandler.HANDLE_TLEFT:
					gui.setCursor(TC_CURSOR);
					// gui.setCursor(TL_CURSOR);
					break;
				case BoxHandler.HANDLE_TCENTER:
					gui.setCursor(TC_CURSOR);
					break;
				case BoxHandler.HANDLE_TRIGHT:
					// gui.setCursor(TR_CURSOR);
					gui.setCursor(TC_CURSOR);
					break;
				case BoxHandler.HANDLE_CLEFT:
					gui.setCursor(CL_CURSOR);
					break;
				case BoxHandler.HANDLE_CRIGHT:
					gui.setCursor(CR_CURSOR);
					break;
				case BoxHandler.HANDLE_BLEFT:
					// gui.setCursor(BL_CURSOR);
					gui.setCursor(BC_CURSOR);
					break;
				case BoxHandler.HANDLE_BCENTER:
					gui.setCursor(BC_CURSOR);
					break;
				case BoxHandler.HANDLE_BRIGHT:
					// gui.setCursor(BR_CURSOR);
					gui.setCursor(BC_CURSOR);
					break;
				default:
					gui.setCursor(handCursor);
					// gui.setCursor(MOVE_CURSOR);
				}
			} else {
				gui.setCursor(handCursor);
			}
		} else {
			gui.setCursor(defaultCursor);
		}
	}

	private GUIBox addBox(int startRow, int endRow) {
		GUIBox newbox = null;
		if (startRow > endRow) {
			int temp = startRow;
			startRow = endRow;
			endRow = temp;
		}
		if (startRow >= 0 && endRow <= content.getRowCount()) {
			if (boxMan.canAddBox(startRow, endRow)) {
				newbox = boxMan.addBox(startRow, endRow);
				if (newbox != null) {
					resizeColumns();
					if (endRow == content.getRowCount()) {
						insertRow(endRow);
						updateLineNums(content.getRowCount() - 1);
					}
				}
			}
		}
		return newbox;
	}

	private static void drawBoxes(Graphics g) {
		GUIBox b = null;
		selectedBox = boxMan.getSelectedBox();
		for (int i = 0; i < boxMan.boxCount(); ++i) {
			b = boxMan.get(i);
			if (b != selectedBox) {
				drawBox(g, b);
			}
		}
		if (selectedBox != null && !(resizingBox || movingBox)) {
			highlightBox(g, selectedBox.getBounds());
		}
	}

	private static void drawBox(Graphics g, GUIBox box) {
		drawBox(g, box.getBounds());
	}

	private static void drawBox(Graphics g, Boundary b) {
		if (b.y1 < b.y2) {
			g.drawRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		} else if (b.y1 > b.y2) {
			g.drawRect(b.x1, b.y2, b.x2 - b.x1, b.y1 - b.y2);
		} else {
			g.drawLine(b.x1, b.y1, b.x2, b.y2);
		}
	}

	private void clearBox(Graphics g, GUIBox box) {
		clearBox(g, box.getBounds());
	}

	private void clearBox(Graphics g, Boundary b) {
		Color c = g.getColor();
		g.setColor(Color.WHITE);
		if (b.y1 < b.y2) {
			g.drawRect(b.x1, b.y1, b.x2 - b.x1, b.y2 - b.y1);
		} else if (b.y1 > b.y2) {
			g.drawRect(b.x1, b.y2, b.x2 - b.x1, b.y1 - b.y2);
		} else {
			g.drawLine(b.x1, b.y1, b.x2, b.y2);
		}
		g.setColor(c);
	}

	private static void highlightBox(Graphics g, Boundary b) {
		int y1 = b.y1;
		int y2 = b.y2;

		if (y1 > y2) {
			int tempY = y1;
			y1 = y2;
			y2 = tempY;
		}

		Color c = g.getColor();
		g.setColor(Color.RED);

		// Top edge
		g.fillRect(b.x1, y1 - 1, b.x2 - b.x1, boxHPixels + boxPixels);
		// Left side edge
		g.fillRect(b.x1 - 1, y1 - 1, boxHPixels + boxPixels, y2 - y1);
		// Right side edge
		g.fillRect(b.x2, y1 - 1, boxHPixels + boxPixels, y2 - y1);
		// Bottom edge
		g.fillRect(b.x1, y2, b.x2 - b.x1, boxHPixels + boxPixels);

		FillHandle handle = boxMan.computeHandle(b);
		if (handle != null) {
			// Top left
			b = handle.topLeft;
			g.fillRect(b.x1, b.y1, 6, 6);
			// Top middle
			b = handle.topCenter;
			g.fillRect(b.x1, b.y1, 6, 6);
			// Top right
			b = handle.topRight;
			g.fillRect(b.x1, b.y1, 6, 6);
			// Bottom left
			b = handle.bottomLeft;
			g.fillRect(b.x1, b.y1, 6, 6);
			// Bottom right
			b = handle.bottomRight;
			g.fillRect(b.x1, b.y1, 6, 6);
			// Bottom middle
			b = handle.bottomCenter;
			g.fillRect(b.x1, b.y1, 6, 6);
		}
		g.setColor(c);
	}

	private void clearHighlightedBox(Graphics g, Boundary b) {
		int y1 = b.y1;
		int y2 = b.y2;

		if (y1 > y2) {
			int tempY = y1;
			y1 = y2;
			y2 = tempY;
		}

		Color c = g.getColor();
		g.setColor(Color.WHITE);

		// Top edge
		g.fillRect(b.x1, y1 - 1, b.x2 - b.x1, boxHPixels + boxPixels);
		// Left side edge
		g.fillRect(b.x1 - 1, y1 - 1, boxHPixels + boxPixels, y2 - y1);
		// Right side edge
		g.fillRect(b.x2, y1 - 1, boxHPixels + boxPixels, y2 - y1);
		// Bottom edge
		g.fillRect(b.x1, y2, b.x2 - b.x1, boxHPixels + boxPixels);

		FillHandle handle = boxMan.computeHandle(b);
		if (handle != null) {
			// Top left
			b = handle.topLeft;
			// System.out.println("top left: " + b);
			g.fillRect(b.x1, b.y1, 6, 6);
			// Top middle
			b = handle.topCenter;
			// System.out.println("top center: " + b);
			g.fillRect(b.x1, b.y1, 6, 6);
			// Top right
			b = handle.topRight;
			// System.out.println("top right: " + b);
			g.fillRect(b.x1, b.y1, 6, 6);
			// Bottom left
			b = handle.bottomLeft;
			// System.out.println("bottom left: " + b);
			g.fillRect(b.x1, b.y1, 6, 6);
			// Bottom right
			b = handle.bottomRight;
			// System.out.println("bottom right: " + b);
			g.fillRect(b.x1, b.y1, 6, 6);
			// Bottom middle
			b = handle.bottomCenter;
			// System.out.println("bottom center: " + b);
			g.fillRect(b.x1, b.y1, 6, 6);
		}
		g.setColor(c);
	}

	private boolean newBox(int x, int y, boolean temp) {
		boolean modified = false;
		if (!temp || Math.abs(newBox.y2 - y) >= drawBoxOffset) {
			int startRow = boxStartRow;
			int endRow = pointToRow(x, y);
			if (startRow == endRow) {
				if (Math.abs(rowTop(startRow) - y) >= getRowHeight() / 2) {
					++endRow;
				}
			} else {
				if (Math.abs(rowTop(endRow) - y) >= getRowHeight() / 2) {
					++endRow;
				}
			}

			// System.out.println("  startRow="+startRow+",endRow="+endRow);
			if (temp) {
				if (startRow >= 0 && endRow >= 0
						&& startRow <= content.getRowCount()
						&& endRow <= content.getRowCount()) {
					boxMan.clearBox(gui.getGraphics(), newBox);
					boxMan.drawBoxes(gui.getGraphics());
					newBox = calcBoxBounds(startRow, endRow, 0);
					boxMan.drawBox(gui.getGraphics(), newBox);
					report("");
				}
			} else {
				if (!drawBox(startRow, endRow, true)) {
					boxMan.clearBox(gui.getGraphics(), newBox);
					boxMan.drawBoxes(gui.getGraphics());
					report("Cannot draw box: Rows[" + (startRow + 1) + ","
							+ (endRow + 1) + "]\n");
				} else {
					modified = true;
				}
				drawingBox = false;
				newBox = null;
			}
		}
		return modified;
	}

	private boolean resizeBox(int x, int y, boolean temp) {
		boolean modified = false;
		if (!temp || Math.abs(resizeBox.y2 - y) >= drawBoxOffset) {
			int startRow = boxStartRow;
			int endRow = pointToRow(x, y);
			if (startRow == endRow) {
				if (Math.abs(rowTop(startRow) - y) >= getRowHeight() / 2) {
					++endRow;
				}
			} else {
				if (Math.abs(rowTop(endRow) - y) >= getRowHeight() / 2) {
					++endRow;
				}
			}
			// System.out.println("  *startRow="+startRow+",endRow="+endRow);

			if (temp) {
				clearHighlightedBox(gui.getGraphics(), resizeBox);
				resizeBox = calcBoxBounds(startRow, endRow, selectedBox
						.height());
				drawBoxes(gui.getGraphics());
				highlightBox(gui.getGraphics(), resizeBox);
			} else {
				if (boxMan.canAddBox(startRow, endRow)) {
					boxMan.removeSelectedBox();
					GUIBox box = addBox(startRow, endRow);
					boxMan.selectBox(box);
					modified = true;
					report("");
				} else {
					report("Cannot resize box\n");
				}
				resizingBox = false;
				resizeBox = null;
				gui.repaint();
			}
		}
		return modified;
	}

	private boolean moveBox(int x, int y, boolean temp) {
		boolean modified = false;
		if (temp) {
			int diff = Math.abs(moveY - y);
			if (diff >= drawBoxOffset) {
				boolean move = false;
				if (moveY > y) {
					if (!isFirstRow(boxStartRow)) {
						--boxStartRow;
						--boxEndRow;
						moveY -= getRowHeight();
						move = true;
					}
				} else {
					if (!isLastRow(boxEndRow - 1)) {
						++boxStartRow;
						++boxEndRow;
						moveY += getRowHeight();
						move = true;
					}
				}
				if (move) {
					clearHighlightedBox(gui.getGraphics(), moveBox);
					moveBox = calcBoxBounds(boxStartRow, boxEndRow, selectedBox
							.height());
					drawBoxes(gui.getGraphics());
					highlightBox(gui.getGraphics(), moveBox);
				}
			}
		} else {
			GUIBox box = boxMan.getSelectedBox();
			boxMan.removeSelectedBox();
			if (boxMan.canAddBox(boxStartRow, boxEndRow)) {
				box = boxMan.addBox(boxStartRow, boxEndRow);
				modified = true;
				report("");
			} else {
				boxMan.addBox(box);
				report("Cannot move box\n");
			}
			boxMan.selectBox(box);
			movingBox = false;
			moveBox = null;
			recalcBoxBounds();
			gui.repaint();
		}
		return modified;
	}

	public void setPoints() {
		tcm.pause();
		point = cursorToPoint(cursor);
		tcm.resume();

		Rectangle visibleRect = gui.getVisibleRect();
		if (point.y < visibleRect.y) {
			visibleRect.translate(0, point.y + Row.getHeight()
					- (visibleRect.y + cursorVOffset));
		} else if (point.y > (visibleRect.y + visibleRect.height)
				- cursorVOffset) {
			visibleRect.translate(0, point.y
					- (visibleRect.y + visibleRect.height) + cursorVOffset);
		}
		if (point.x < visibleRect.x + cursorHOffset) {
			visibleRect.translate(point.x - (visibleRect.x + cursorHOffset), 0);
		} else if (point.x > (visibleRect.x + visibleRect.width)
				- cursorHOffset) {
			visibleRect.translate(point.x - (visibleRect.x + visibleRect.width)
					+ cursorHOffset, 0);
		}
		gui.scrollRectToVisible(visibleRect);
	}

	private static Point cursorToPoint(TextCursor c) {
		return cursorToPoint(c.col, c.row, c.offset);
	}

	private static Point cursorToPoint(int col, int row, int offset) {
		int x = margin.left;
		for (int i = 0; i < col; ++i) {
			x += controller.getColWidth(i) + controller.getColSpacing(i, i + 1);
		}
		x += Row.getLPad() + (offset * drawer.getCharWidth());

		int y = margin.top + (row * getRowHeight());

		return new Point(x, y);
	}

	private int rowTop(int row) {
		return margin.top + (row * getRowHeight());
	}

	private TextCursor pointToCursor(int x, int y) {
		TextCursor tc = new TextCursor();
		tc.row = pointToRow(x, y);
		tc.col = pointToCol(x, y);
		tc.offset = 0;
		Point tempP = cursorToPoint(tc);
		tc.offset = (x - tempP.x + (drawer.getCharWidth() / 2))
				/ drawer.getCharWidth();

		return tc;
	}

	private int pointToRow(int x, int y) {
		if (y > margin.top) {
			return (y - margin.top) / getRowHeight();
		} else {
			return -1;
		}
	}

	private int pointToCol(int x, int y) {
		x -= margin.left;
		int col = -1;
		int width = 0;
		while (x > width) {
			++col;
			width += (getColWidth(col) + getColSpacing(col, col + 1));
		}
		return col;
	}

	private boolean cursorOk(TextCursor tc) {
		boolean ok = true;
		if (tc.col <= 0 || tc.col > content.getColumnCount() - 1 || tc.row < 0
				|| tc.row >= content.getRowCount() || tc.offset < 0
				|| tc.offset > content.getCol(tc.col).getRow(tc.row).length()) {

			ok = false;
		}
		return ok;
	}

	private TextCursor fixCursor(TextCursor tc) {
		TextCursor newTc = (TextCursor) tc.clone();
		if (newTc.col <= 0 || newTc.col > content.getColumnCount() - 1) {
			newTc.col = getFormulaColumn();
			newTc.offset = 0;
		}
		if (newTc.row < 0) {
			newTc.row = 0;
		} else if (newTc.row >= content.getRowCount()) {
			newTc.row = content.getRowCount() - 1;
		}
		if (newTc.offset < 0
				|| newTc.offset > content.getCol(newTc.col).getRow(newTc.row)
						.length()) {
			newTc.offset = content.getCol(newTc.col).getRow(newTc.row).length();
		}
		return newTc;
	}

	protected void insertRow(int position) {
		content.insertRow(position);
		if (position < content.getRowCount() - 1) {
			boxMan.insertRow(position - 1);
		} else {
			boxMan.insertRow(position);
		}
		// updateLineRefs(position, content.getRowCount()-1, 1);
		updateLineNums(position);

		if (content.getRowCount() > MIN_VIEW_ROWS) {
			resizeGUIHeight(true);
		}
	}

	protected void removeRow(int row) {
		content.removeRow(row);
		boxMan.removeRow(row);

		updateLineNums(row);
		updateLineRefs(row, content.getRowCount() - 1, -1);

		if (content.getRowCount() > MIN_VIEW_ROWS) {
			resizeGUIHeight(true);
		}
		recalcBoxBounds();
	}

	protected static int getColHeight() {
		return gui.getPreferredSize().height - (margin.top + margin.bottom);
	}

	protected abstract int getColWidth(int col);

	protected abstract int getColSpacing(int col1, int col2);

	protected static int getRowHeight() {
		return Row.getHeight() + ((boxPixels + boxHPixels) * 2) + rowSpacing;
	}

	protected int getBoxHeight(int rows) {
		if (rows <= 0) {
			return 0;
		} else {
			return rows * getRowHeight() - rowSpacing - (boxHPixels * 2);
		}
	}

	protected void printWidths() {
		for (int i = 0; i < content.getColumnCount(); ++i) {
			System.out.println("col(" + i + ").width: " + getColWidth(i));
		}
	}

	protected void resizeHeight(int pixels) {
		Dimension d = gui.getPreferredSize();
		gui.setPreferredSize(new Dimension(d.width, d.height + pixels));
		gui.revalidate();
	}

	protected abstract void resizeColumns();

	protected abstract void resizeColumn(int col, int pixels);

	protected void recalcBoxBounds() {
		for (int i = 0; i < boxMan.boxCount(); ++i) {
			GUIBox box = boxMan.get(i);
			box.setBounds(calcBoxBounds(box.startRow(), box.endRow(), box
					.height()));
		}
	}

	protected void goToRow(int row) {
		cursor.row = row;
	}

	protected void goToPrevRow() {
		--(cursor.row);
	}

	protected void goToNextRow() {
		++(cursor.row);
	}

	protected void goToFirstRow() {
		cursor.row = 0;
	}

	protected void goToLastRow() {
		cursor.row = content.getRowCount() - 1;
	}

	protected boolean isFirstRow(int row) {
		return (row == 0);
	}

	protected boolean isLastRow(int row) {
		return (row + 1 >= content.getRowCount());
	}

	protected void goToCol(int col) {
		cursor.col = col;
	}

	protected void goToPrevCol() {
		--(cursor.col);
	}

	protected void goToNextCol() {
		++(cursor.col);
	}

	protected abstract void goToFormulaCol();

	protected abstract void goToFirstCol();

	protected abstract void goToLastCol();

	protected abstract boolean isFirstCol(int col);

	protected abstract boolean isLastCol(int col);

	protected abstract void updateLineNums(int startRow);

	protected abstract void updateLineNums(int startRow, int endRow);

	protected void updateLineRefs(int startRow, int endRow, int add) {
		// System.out.println(" **updateLineRefs: start="+startRow+",end="+endRow+",add="+add);
		Col col = content.getCol(getLinerefColumn());
		for (int i = startRow; i <= endRow; ++i) {
			Row row = col.getRow(i);
			// System.out.println(" updateLineRefs: row: "+i+", " + row.length()
			// + ", "+row);
			String lineref = row.toString().trim();
			if (lineref.length() > 0) {
				row.deleteAll();
				String[] linerefs = lineref.split(",");
				for (int k = 0; k < linerefs.length; ++k) {
					lineref = linerefs[k];
					try {
						LineReference[] lrArr = lineParser.parse(lineref);
						LineReference lr = lrArr[0];
						if (add < 0) {
							if (lr.getBegin() == startRow + 1) {
								row.append('?');
							} else if (lr.getBegin() > startRow + 1) {
								row.append(Integer.toString(lr.getBegin() - 1));
							} else {
								row.append(Integer.toString(lr.getBegin()));
							}
							if (lr instanceof Range) {
								row.append('-');
								if (lr.getEnd() == startRow + 1) {
									row.append('?');
								} else if (lr.getEnd() > startRow + 1) {
									row.append(Integer
											.toString(lr.getEnd() - 1));
								} else {
									row.append(Integer.toString(lr.getEnd()));
								}
							}
						} else {
							if (lr.getBegin() >= startRow + 1) {
								row.append(Integer.toString(lr.getBegin() + 1));
							} else {
								row.append(Integer.toString(lr.getBegin()));
							}
							if (lr instanceof Range) {
								row.append('-');

								if (lr.getEnd() >= startRow + 1) {
									row.append(Integer
											.toString(lr.getEnd() + 1));
								} else {
									row.append(Integer.toString(lr.getEnd()));
								}
							}
						}
					} catch (LineRefParserException lrpe) {
						row.append(lineref);
					}
					row.append(',');
				}
				row.deleteCharAt(row.length() - 1);
			}
		}
	}

	public abstract RawProof getProof();

	protected abstract Boundary calcBoxBounds(int startRow, int endRow,
			int height);

	static class TextCursorManager implements Runnable {
		private boolean on;
		private boolean paused;
		private boolean running;

		TextCursorManager() {
			on = false;
			paused = true;
			running = false;
		}

		public void start() {
			if (!running) {
				new Thread(this).start();
				running = true;
			}
		}

		public void run() {
			while (true) {
				try {
					Thread.sleep(500);
				} catch (Exception e) {

				}
				if (!paused) {
					if (on) {
						clearCursor();
					} else {
						drawCursor();
					}
					on = !on;
				}
			}
		}

		public boolean isPaused() {
			return paused;
		}

		public boolean isRunning() {
			return running;
		}

		public void pause() {
			paused = true;
			clearCursor();
		}

		public void resume() {
			paused = false;
		}

		private void drawCursor() {
			Graphics g = gui.getGraphics();
			if (g != null) {
				Color c = g.getColor();
				g.setColor(Color.BLACK);
				int x = point.x;
				int y = point.y + boxPixels + boxHPixels;
				g.drawLine(x, y, x, y + Row.getHeight());
				g.setColor(c);
			}
		}

		private void clearCursor() {
			Graphics g = gui.getGraphics();
			if (g != null) {
				Color c = g.getColor();
				g.setColor(Color.WHITE);
				int x = point.x;
				int y = point.y + boxPixels + boxHPixels;
				g.drawLine(x, y, x, y + Row.getHeight());
				g.setColor(c);
			}
		}
	}

	static class TextCursor {
		int col;
		int row;
		int offset;

		TextCursor() {
			this(0, 0, 0);
		}

		TextCursor(int col, int row, int offset) {
			this.col = col;
			this.row = row;
			this.offset = offset;
		}

		public String toString() {
			return "cursor:col=" + col + ",row=" + row + ",off=" + offset;
		}

		public Object clone() {
			return new TextCursor(col, row, offset);
		}
	}

	static class Margin {
		int top;
		int left;
		int right;
		int bottom;

		public Margin() {
			this(0, 0, 0, 0);
		}

		public Margin(int top, int left, int right, int bottom) {
			this.top = top;
			this.left = left;
			this.right = right;
			this.bottom = bottom;
		}

		public int sumHorizontal() {
			return left + right;
		}

		public int sumVertical() {
			return top + bottom;
		}
	}
}
