/*
 * OutputFile.java       Jan 19, 2007
 *
 * Copyright (c) 2007, Brian Mulanda, all rights reserved.
 */

package edu.ksu.cis.ndpc.io;

import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.PrintStream;

/**
 * Represents an file opened for output
 * 
 * @author Brian Mulanda (<a
 *         href="mailto:bmulanda@ksu.edu">bmulanda@ksu.edu</a>)
 * 
 * @see InputFile
 * @see IO
 * 
 */

public class OutputFile {
	private FileOutputStream fos;
	private File f;

	public OutputFile(File f) throws IOException {
		this(f, false);
	}

	public OutputFile(File f, boolean append) throws IOException {
		try {
			this.f = f;
			fos = new FileOutputStream(f, append);
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}
	}

	public File getFile() {
		return f;
	}

	public void writeObject(Object o) throws IOException {
		ObjectOutputStream oos = new ObjectOutputStream(fos);
		oos.writeObject(o);
		oos.flush();
	}

	public void writeSting(String s) throws IOException {
		PrintStream out = new PrintStream(fos);
		out.print(s);
		out.flush();
	}

	public void close() {
		finalize();
	}

	public void finalize() {
		try {
			fos.close();
		} catch (IOException ioe) {
		} finally {
			fos = null;
		}
	}
}