package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleEqualsIntro extends Rule {
	private static String name = getDescription(Rule.EQUAL_INTRO);

	public RuleEqualsIntro() {
		super();
	}

	public int getType() {
		return Rule.EQUAL_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		/*
		 * if ( params == null || params.length != 1 || params[0] == null ) {
		 * throw new RuleException("Rule requires one formula"); } else if ( !
		 * (params[0] instanceof ProofStatement) ) { throw new
		 * RuleException("Rule requires one formula"); }
		 */
		ProofStatement conclusion = (ProofStatement) params[0];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_EQUAL) {
			throw new RuleException("Formula " + conclusion
					+ " must have '=' as its main connective");
		} else {
			ParseTree leftSubtree = conclusion.getFormula().getParseTree()
					.getLeftSubtree();
			ParseTree rightSubtree = conclusion.getFormula().getParseTree()
					.getRightSubtree();
			if (!leftSubtree.equals(rightSubtree)) {
				throw new RuleException(
						"The left side of the main connective '=' in "
								+ "formula " + conclusion
								+ " must match the right side "
								+ "of the main connective '='");
			}
		}
	}
}