package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofChecker;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleExistsIntro extends Rule {
	private static String name = getDescription(Rule.EXIST_INTRO);

	public RuleExistsIntro() {
		super();
	}

	public int getType() {
		return Rule.EXIST_INTRO;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires a formula");
		} else if (!((params[0] instanceof ProofStatement) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires a formula");
		}

		ProofStatement premise = (ProofStatement) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		Symbol root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_EXISTS) {
			throw new RuleException(
					"Formula "
							+ conclusion
							+ " must be existentially quantified "
							+ "with the existential symbol and a variable. For example Ex(formula)");
		}

		ParseTree premiseTree = premise.getFormula().getParseTree();
		ParseTree conclusionTree = conclusion.getFormula().getParseTree();

		if (!premiseTree.equals(conclusionTree.getSubtree())) {
			Symbol var = new Symbol(conclusionTree.getRootNode().getValue()
					.substring(1), LangSymbols.S_VAR);
			ParseTree termX = new ParseTree(var);

			ParseTree termT = ProofChecker.findTermT(conclusionTree
					.getSubtree(), premiseTree, termX);

			ParseTree testConclTree = ProofChecker.substitute(
					(ParseTree) conclusionTree.getSubtree().clone(), termX,
					termT);

			if (!testConclTree.equals(premiseTree)) {
				String formula = conclusion.getFormula().getFormulaString()
						.substring(0, conclusionTree.getRootNode().getLength())
						.trim();
				throw new RuleException("Incorrect substitution. From "
						+ conclusion + ", you should " + "be able to obtain "
						+ premise + " by first " + "removing " + formula
						+ ", and then replacing all free occurrences "
						+ "of the variable " + var + " with the term "
						+ termT.allToString() + ".");
			}
		}
	}
}
