package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleImpElim extends Rule {
	private static String name = getDescription(Rule.IMP_ELIM);

	public RuleImpElim() {
		super();
	}

	public int getType() {
		return Rule.IMP_ELIM;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 3 || params[0] == null
				|| params[1] == null || params[2] == null) {
			throw new RuleException("Rule requires two formulas");
		} else if (!((params[0] instanceof ProofStatement)
				&& (params[1] instanceof ProofStatement) && (params[2] instanceof ProofStatement))) {
			throw new RuleException("Rule requires two formulas");
		}

		ProofStatement premise1 = (ProofStatement) params[0];
		ProofStatement premise2 = (ProofStatement) params[1];
		ProofStatement conclusion = (ProofStatement) params[2];

		Symbol root = premise1.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_IMPL) {
			throw new RuleException("Formula " + premise1
					+ " must have '->' as its main connective");
		}

		ParseTree leftSubtree = premise1.getFormula().getParseTree()
				.getLeftSubtree();
		ParseTree rightSubtree = premise1.getFormula().getParseTree()
				.getRightSubtree();
		if (!leftSubtree.equals(premise2.getFormula().getParseTree())) {
			throw new RuleException("Formula " + premise2
					+ " must be the same as the "
					+ "left side of the main connective '->' in formula"
					+ premise1);
		} else if (!rightSubtree.equals(conclusion.getFormula().getParseTree())) {
			throw new RuleException("Formula " + conclusion
					+ " must be the same as the "
					+ "right side of the main connective '->' in formula"
					+ premise1);
		}
	}
}