package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RuleMT extends Rule {
	private static String name = getDescription(Rule.MT);

	public RuleMT() {
		super();
	}

	public int getType() {
		return Rule.MT;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 3 || params[0] == null
				|| params[1] == null || params[2] == null) {
			throw new RuleException("Rule requires two formulas");
		} else if (!((params[0] instanceof ProofStatement)
				&& (params[1] instanceof ProofStatement) && (params[2] instanceof ProofStatement))) {
			throw new RuleException("Rule requires two formulas");
		}

		ProofStatement premise1 = (ProofStatement) params[0];
		ProofStatement premise2 = (ProofStatement) params[1];
		ProofStatement conclusion = (ProofStatement) params[2];

		Symbol root = premise1.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_IMPL) {
			throw new RuleException("Formula " + premise1
					+ " must have '->' as its main connective");
		}

		root = premise2.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + premise2
					+ " must be negated (begin with ~) like this: "
					+ " ~formula or ~(formula)");
		}

		root = conclusion.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("Formula " + conclusion
					+ " must be negated (begin with ~) like this: "
					+ " ~formula or ~(formula)");
		}

		ParseTree p1LeftSubtree = premise1.getFormula().getParseTree()
				.getLeftSubtree();
		ParseTree p1RightSubtree = premise1.getFormula().getParseTree()
				.getRightSubtree();
		ParseTree p2Subtree = premise2.getFormula().getParseTree().getSubtree();
		ParseTree cSubtree = conclusion.getFormula().getParseTree()
				.getSubtree();
		if (!p1LeftSubtree.equals(cSubtree)) {
			throw new RuleException("The formula in " + conclusion
					+ ", without negation, "
					+ "should match the formula on the left side of the main "
					+ "connective '->' in formula " + premise1);
		} else if (!p1RightSubtree.equals(p2Subtree)) {
			throw new RuleException("The formula in " + premise2
					+ ", without negation, "
					+ "should match the formula on the right side of the main "
					+ "connective '->' in formula " + premise1);
		}
	}
}