package edu.ksu.cis.ndpc.rules;

import edu.ksu.cis.ndpc.BoxedStatements;
import edu.ksu.cis.ndpc.LangSymbols;
import edu.ksu.cis.ndpc.ParseTree;
import edu.ksu.cis.ndpc.ProofStatement;
import edu.ksu.cis.ndpc.Symbol;

/**
 * @author Brian Mulanda
 * 
 */

public final class RulePBC extends Rule {
	private static String name = getDescription(Rule.PBC);

	public RulePBC() {
		super();
	}

	public int getType() {
		return Rule.PBC;
	}

	public void check(RuleParameter[] params) throws RuleException {
		if (params == null || params.length != 2 || params[0] == null
				|| params[1] == null) {
			throw new RuleException("Rule requires one set of boxed formulas");
		} else if (!((params[0] instanceof BoxedStatements) && (params[1] instanceof ProofStatement))) {
			throw new RuleException("Rule requires one set of boxed formulas");
		}

		BoxedStatements premise = (BoxedStatements) params[0];
		ProofStatement conclusion = (ProofStatement) params[1];

		ProofStatement firstFormula = premise.getFirst();
		if (firstFormula.getRule().type != Rule.ASSUMPTION) {
			throw new RuleException("The first formula " + firstFormula + " "
					+ "in box " + premise.getRange() + " must be an "
					+ "assumption");
		}

		Symbol root = firstFormula.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_NEG) {
			throw new RuleException("The first formula " + firstFormula
					+ " in " + "box: " + premise.getRange()
					+ " must be negated "
					+ "(begin with a '~') like this: ~formula or ~(formula)");
		}

		ProofStatement lastFormula = premise.getLast();
		root = lastFormula.getFormula().getParseTree().getRootNode();
		if (root.getType() != LangSymbols.S_BOTTOM) {
			throw new RuleException("The last formula in box: "
					+ premise.getRange() + "must be bottom '_|_'");
		}

		ParseTree ffSubtree = firstFormula.getFormula().getParseTree()
				.getSubtree();

		if (!ffSubtree.equals(conclusion.getFormula().getParseTree())) {
			throw new RuleException("The formula in " + conclusion
					+ " must match " + "the first formula " + firstFormula
					+ " in the box: " + premise.getRange()
					+ ", but without the negation ");
		}
	}
}